@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -73,6 +73,7 @@ class Msg {
   static GmshMessage *GetCallback();
   static void Barrier();
   static int GetNumThreads();
+  static void SetNumThreads(int num);
   static int GetMaxThreads();
   static int GetThreadNum();
   static void SetVerbosity(int val);
@@ -151,4 +152,19 @@ class Msg {
   static void ImportPhysicalGroupsInOnelab();
 };
 
+// a class to print the progression and estimated remaining time
+class MsgProgressStatus
+{
+private:
+  int _totalElementToTreat, _currentI, _nextIToCheck;
+  double _initialTime, _lastTime;
+  int _lastPercentage;
+  int _progressMeterStep;
+public:
+  MsgProgressStatus(int numElementToTreat);
+  ~MsgProgressStatus();
+  void setInitialTime(double time) { _initialTime = time; }
+  void next();
+};
+
 #endif
