@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -13,8 +13,8 @@ typedef unsigned long intptr_t;
 #endif
 #include "GmshMessage.h"
 #include "GmshIO.h"
+#include "GModelIO_GEO.h"
 #include "Options.h"
-#include "Geo.h"
 #include "GModel.h"
 #include "Numeric.h"
 #include "Context.h"
@@ -158,22 +158,6 @@ void AddToTemporaryBoundingBox(double x, double y, double z)
   for(int i = 0; i < 3; i++) CTX::instance()->cg[i] = temp_bb.center()[i];
 }
 
-static void ComputeMaxEntityNum()
-{
-  GModel::current()->getGEOInternals()->MaxPointNum =
-    std::max(GModel::current()->getGEOInternals()->MaxPointNum,
-             GModel::current()->getMaxElementaryNumber(0));
-  GModel::current()->getGEOInternals()->MaxLineNum =
-    std::max(GModel::current()->getGEOInternals()->MaxLineNum,
-             GModel::current()->getMaxElementaryNumber(1));
-  GModel::current()->getGEOInternals()->MaxSurfaceNum =
-    std::max(GModel::current()->getGEOInternals()->MaxSurfaceNum,
-             GModel::current()->getMaxElementaryNumber(2));
-  GModel::current()->getGEOInternals()->MaxVolumeNum =
-    std::max(GModel::current()->getGEOInternals()->MaxVolumeNum,
-             GModel::current()->getMaxElementaryNumber(3));
-}
-
 static std::vector<gmshFILE> openedFiles;
 
 int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
@@ -429,6 +413,10 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setBoundingB
   else if(ext == ".dat" || ext == ".DAT"){
     if(!strncmp(header, "BEGIN ACTRAN", 12))
       status = GModel::current()->readACTRAN(fileName);
+    else if(!strncmp(header, "!", 1) ||
+            !strncmp(header, ";ECHO", 5) ||
+            !strncmp(header, ".NOE", 4))
+      status = GModel::current()->readSAMCEF(fileName);
     else
       status = GModel::current()->readBDF(fileName);
   }
@@ -521,7 +509,19 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setBoundingB
     }
   }
 
-  ComputeMaxEntityNum();
+  GModel::current()->getGEOInternals()->setMaxTag
+    (0, std::max(GModel::current()->getGEOInternals()->getMaxTag(0),
+                 GModel::current()->getMaxElementaryNumber(0)));
+  GModel::current()->getGEOInternals()->setMaxTag
+    (1, std::max(GModel::current()->getGEOInternals()->getMaxTag(1),
+                 GModel::current()->getMaxElementaryNumber(1)));
+  GModel::current()->getGEOInternals()->setMaxTag
+    (2, std::max(GModel::current()->getGEOInternals()->getMaxTag(2),
+                 GModel::current()->getMaxElementaryNumber(2)));
+  GModel::current()->getGEOInternals()->setMaxTag
+    (3, std::max(GModel::current()->getGEOInternals()->getMaxTag(3),
+                 GModel::current()->getMaxElementaryNumber(3)));
+
   if(setBoundingBox) SetBoundingBox();
   CTX::instance()->geom.draw = 1;
   CTX::instance()->mesh.changed = ENT_ALL;
@@ -593,10 +593,7 @@ int MergePostProcessingFile(const std::string &fileName, int showViews,
     GModel *m = new GModel();
     GModel::setCurrent(m);
   }
-  // FIXME: disabled onelab physical group import for now, as the number of
-  // groups in mesh-based post-pro files can be different from the # in the
-  // model, which will trigger setChanged(Gmsh), leading undesirable remeshing
-  int ret = MergeFile(fileName, warnIfMissing, old->bounds().empty() ? true : false, false);
+  int ret = MergeFile(fileName, warnIfMissing, old->bounds().empty() ? true : false);
   GModel::setCurrent(old);
   old->setVisibility(1);
 
@@ -643,6 +640,8 @@ void ClearProject()
 #if defined(HAVE_PARSER)
   gmsh_yysymbols.clear();
   gmsh_yystringsymbols.clear();
+  gmsh_yyfactory.clear();
+  gmsh_yynamespaces.clear();
 #endif
   for(int i = GModel::list.size() - 1; i >= 0; i--)
     delete GModel::list[i];
@@ -656,7 +655,8 @@ void ClearProject()
   Msg::Info("Done clearing all models and views");
 
   new GModel();
-  GModel::current()->setFileName(CTX::instance()->defaultFileName);
+  std::string base = (getenv("PWD") ? "" : CTX::instance()->homeDir);
+  GModel::current()->setFileName(base + CTX::instance()->defaultFileName);
   GModel::current()->setName("");
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
@@ -704,6 +704,8 @@ void OpenProject(const std::string &fileName)
   for(std::map<std::string, std::string>::iterator it = cls.begin();
       it != cls.end(); it++)
     gmsh_yystringsymbols[it->first] = std::vector<std::string>(1, it->second);
+  gmsh_yyfactory.clear();
+  gmsh_yynamespaces.clear();
   FunctionManager::Instance()->clear();
 #endif
 
