@@ -343,6 +343,7 @@ double opt_general_light50(OPT_ARGS_NUM);
 double opt_general_light51(OPT_ARGS_NUM);
 double opt_general_light52(OPT_ARGS_NUM);
 double opt_general_light53(OPT_ARGS_NUM);
+double opt_general_num_threads(OPT_ARGS_NUM);
 double opt_geometry_transform(OPT_ARGS_NUM);
 double opt_geometry_transform00(OPT_ARGS_NUM);
 double opt_geometry_transform01(OPT_ARGS_NUM);
@@ -550,6 +551,9 @@ double opt_mesh_clip(OPT_ARGS_NUM);
 double opt_mesh_ignore_part_bound(OPT_ARGS_NUM);
 double opt_mesh_ignore_periodicity(OPT_ARGS_NUM);
 double opt_mesh_preserve_numbering_msh2(OPT_ARGS_NUM);
+double opt_mesh_max_num_threads_1d(OPT_ARGS_NUM);
+double opt_mesh_max_num_threads_2d(OPT_ARGS_NUM);
+double opt_mesh_max_num_threads_3d(OPT_ARGS_NUM);
 double opt_solver_listen(OPT_ARGS_NUM);
 double opt_solver_timeout(OPT_ARGS_NUM);
 double opt_solver_plugins(OPT_ARGS_NUM);
