@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -11,6 +11,7 @@
 #include <FL/Fl_Shared_Image.H>
 #include <FL/Fl_File_Icon.H>
 #include <FL/fl_draw.H>
+#include <FL/fl_ask.H>
 #include "FlGui.h"
 #include "drawContextFltk.h"
 #include "drawContextFltkCairo.h"
@@ -52,13 +53,41 @@
 #endif
 
 // check (now!) if there are any pending events, and process them
-void FlGui::check(){ Fl::check(); }
+void FlGui::check()
+{
+  if(Msg::GetThreadNum() == 0) Fl::check();
+}
 
 // wait (possibly indefinitely) for any events, then process them
-void FlGui::wait(){ Fl::wait(); }
+void FlGui::wait()
+{
+  if(Msg::GetThreadNum() == 0) Fl::wait();
+}
 
 // wait (at most time seconds) for any events, then process them
-void FlGui::wait(double time){ Fl::wait(time); }
+void FlGui::wait(double time)
+{
+  if(Msg::GetThreadNum() == 0) Fl::wait(time);
+}
+
+void FlGui::lock()
+{
+#if defined(_OPENMP)
+  if(Msg::GetThreadNum() > 0){
+    Fl::lock();
+  }
+#endif
+}
+
+void FlGui::unlock()
+{
+#if defined(_OPENMP)
+  if(Msg::GetThreadNum() > 0){
+    Fl::unlock();
+    //Fl::awake();
+  }
+#endif
+}
 
 void FlGui::setOpenedThroughMacFinder(const std::string &name)
 {
@@ -300,6 +329,11 @@ FlGui::FlGui(int argc, char **argv)
   Fl::error = error_handler;
   Fl::fatal = fatal_error_handler;
 
+#if defined(_OPENMP)
+  // tell fltk we're in multi-threaded mode
+  Fl::lock();
+#endif
+
   // set X display
   if(CTX::instance()->display.size())
     Fl::display(CTX::instance()->display.c_str());
@@ -335,6 +369,8 @@ FlGui::FlGui(int argc, char **argv)
   // use retina resolution if available
 #if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3) && (FL_PATCH_VERSION >= 4)
   Fl::use_high_res_GL(CTX::instance()->highResolutionGraphics);
+#elif (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION >= 4)
+  Fl::use_high_res_GL(CTX::instance()->highResolutionGraphics);
 #endif
 
   // register image formats not in core fltk library (jpeg/png)
@@ -363,6 +399,9 @@ FlGui::FlGui(int argc, char **argv)
   fl_mac_set_about(help_about_cb, 0);
 #endif
 
+  // don't move input dialogs to follow mouse
+  fl_message_hotspot(0);
+
   // create main graphic window (note that we create all the windows even if
   // some are not displayed, since the shortcuts should be valid even for hidden
   // windows, and we don't want to test for widget existence every time)
@@ -443,6 +482,7 @@ FlGui::FlGui(int argc, char **argv)
   clipping = new clippingWindow(CTX::instance()->deltaFontSize);
   manip = new manipWindow(CTX::instance()->deltaFontSize);
   elementaryContext = new elementaryContextWindow(CTX::instance()->deltaFontSize);
+  transformContext = new transformContextWindow(CTX::instance()->deltaFontSize);
   physicalContext = new physicalContextWindow(CTX::instance()->deltaFontSize);
   meshContext = new meshContextWindow(CTX::instance()->deltaFontSize);
   help = new helpWindow();
@@ -454,6 +494,11 @@ FlGui::FlGui(int argc, char **argv)
   for(unsigned int i = 0; i < graph.size(); i++)
     for(unsigned int j = 0; j < graph[i]->gl.size(); j++)
       graph[i]->gl[j]->redraw();
+
+  if(CTX::instance()->showOptionsOnStartup)
+    options->win->show();
+  if(CTX::instance()->showMessagesOnStartup)
+    graph[0]->showMessages();
 }
 
 FlGui *FlGui::_instance = 0;
@@ -464,7 +509,6 @@ bool FlGui::available()
   return (_instance != 0);
 }
 
-
 FlGui *FlGui::instance(int argc, char **argv)
 {
   if(!_instance){
@@ -590,6 +634,36 @@ int FlGui::testGlobalShortcuts(int event)
         graph[i]->gl[j]->invertSelection = 1;
     status = 0; // trick: do as if we didn't use it
   }
+  else if(Fl::test_shortcut('x')) {
+    elementaryContext->butt[0]->value(!elementaryContext->butt[0]->value());
+    status = 1; // trick: do as if we didn't use it
+  }
+  else if(Fl::test_shortcut('y')) {
+    elementaryContext->butt[1]->value(!elementaryContext->butt[1]->value());
+    status = 1; // trick: do as if we didn't use it
+  }
+  else if(Fl::test_shortcut('z')) {
+    elementaryContext->butt[2]->value(!elementaryContext->butt[2]->value());
+    status = 1; // trick: do as if we didn't use it
+  }
+  else if(Fl::test_shortcut(FL_SHIFT + 'x')) {
+    elementaryContext->butt[0]->value(0);
+    elementaryContext->butt[1]->value(1);
+    elementaryContext->butt[2]->value(1);
+    status = 1; // trick: do as if we didn't use it
+  }
+  else if(Fl::test_shortcut(FL_SHIFT + 'y')) {
+    elementaryContext->butt[0]->value(1);
+    elementaryContext->butt[1]->value(0);
+    elementaryContext->butt[2]->value(1);
+    status = 1; // trick: do as if we didn't use it
+  }
+  else if(Fl::test_shortcut(FL_SHIFT + 'z')) {
+    elementaryContext->butt[0]->value(1);
+    elementaryContext->butt[1]->value(1);
+    elementaryContext->butt[2]->value(0);
+    status = 1; // trick: do as if we didn't use it
+  }
   else if(Fl::test_shortcut(FL_Escape) ||
           Fl::test_shortcut(FL_META + FL_Escape) ||
           Fl::test_shortcut(FL_SHIFT + FL_Escape) ||
@@ -861,13 +935,13 @@ int FlGui::testArrowShortcuts()
     return 1;
   }
 #if defined(HAVE_POPPLER)
-  else if(Fl::test_shortcut('u')) {
+  else if(Fl::test_shortcut('u') || Fl::test_shortcut(FL_Page_Up)) {
     gmshPopplerWrapper::setCurrentPageDown();
     drawContext::global()->draw();
     drawContext::global()->draw();
     return 1;
   }
-  else if(Fl::test_shortcut('d')) {
+  else if(Fl::test_shortcut('d') || Fl::test_shortcut(FL_Page_Down)) {
     gmshPopplerWrapper::setCurrentPageUp();
     drawContext::global()->draw();
     drawContext::global()->draw();
@@ -1210,6 +1284,8 @@ void window_cb(Fl_Widget *w, void *data)
       FlGui::instance()->fields->win->show();
     if(FlGui::instance()->elementaryContext->win->shown())
       FlGui::instance()->elementaryContext->win->show();
+    if(FlGui::instance()->transformContext->win->shown())
+      FlGui::instance()->transformContext->win->show();
     if(FlGui::instance()->physicalContext->win->shown())
       FlGui::instance()->physicalContext->win->show();
     if(FlGui::instance()->meshContext->win->shown())
@@ -1229,8 +1305,9 @@ void window_cb(Fl_Widget *w, void *data)
 
 void FlGui::addMessage(const char *msg)
 {
-  for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++)
+  for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++){
     FlGui::instance()->graph[i]->addMessage(msg);
+  }
 }
 
 void FlGui::saveMessages(const char *fileName)
