@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -31,6 +31,7 @@ class highOrderToolsWindow;
 class clippingWindow;
 class manipWindow;
 class elementaryContextWindow;
+class transformContextWindow;
 class physicalContextWindow;
 class meshContextWindow;
 class helpWindow;
@@ -69,6 +70,7 @@ class FlGui{
   clippingWindow *clipping;
   manipWindow *manip;
   elementaryContextWindow *elementaryContext;
+  transformContextWindow *transformContext;
   physicalContextWindow *physicalContext;
   meshContextWindow *meshContext;
   helpWindow *help;
@@ -89,6 +91,9 @@ class FlGui{
   static void wait();
   // wait (at most time seconds) for any events, then process them
   static void wait(double time);
+  // lock/unlock child threads
+  static void lock();
+  static void unlock();
   // is a file opened through the Mac Finder?
   static void setOpenedThroughMacFinder(const std::string &name);
   static std::string getOpenedThroughMacFinder();
