@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -13,19 +13,36 @@
 #include <FL/Fl_Check_Button.H>
 #include <FL/Fl_Choice.H>
 #include <FL/Fl_Group.H>
+#include <FL/Fl_Tabs.H>
 
 class elementaryContextWindow{
+ public:
+  Fl_Window *win;
+  Fl_Tabs *tab1, *tab2;
+  Fl_Input *input[80];
+  Fl_Value_Input *value[10];
+  Fl_Group *group[20];
+  Fl_Check_Button *butt[3];
+ public:
+  elementaryContextWindow(int deltaFontSize=0);
+  void show(int pane);
+  void hide(){ win->hide(); }
+  void updatePoint(double pt[3], int which);
+  bool frozenPointCoord(int coord);
+};
+
+class transformContextWindow{
  public:
   Fl_Window *win;
   Fl_Input *input[30];
   Fl_Value_Input *value[10];
   Fl_Group *group[10];
-  Fl_Check_Button *_butt[3];
+  Fl_Check_Button *butt[20];
+  Fl_Choice *choice;
  public:
-  elementaryContextWindow(int deltaFontSize=0);
-  void show(int pane);
-  void updatePoint(double pt[3]);
-  bool frozenPointCoord(int coord);
+  transformContextWindow(int deltaFontSize=0);
+  void show(int pane, bool extrude=false, bool selection=true);
+  void hide(){ win->hide(); }
 };
 
 class physicalContextWindow{
@@ -40,6 +57,7 @@ class physicalContextWindow{
 public:
   physicalContextWindow(int deltaFontSize=0);
   void show(bool remove);
+  void hide(){ win->hide(); }
 };
 
 class meshContextWindow{
@@ -51,6 +69,7 @@ class meshContextWindow{
  public:
   meshContextWindow(int deltaFontSize=0);
   void show(int pane);
+  void hide(){ win->hide(); }
 };
 
 #endif
