@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -127,7 +127,12 @@ class drawContextFltk : public drawContextGlobal{
       else if(w <= 1440) return 12;
       else if(w <= 1680) return 13;
       else if(w <= 1920) return 14;
-      else               return 15;
+      return (w - 1920) / 160 + 15;
+      /*
+      float dpih, dpiv;
+      Fl::screen_dpi(dpih, dpiv);
+      return (int)(dpih / 8.);
+      */
     }
   }
   void setFont(int fontid, int fontsize)
