@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -73,8 +73,10 @@ openglWindow::openglWindow(int x, int y, int w, int h)
   for(int i = 0; i < 4; i++) _trySelectionXYWH[i] = 0;
   _lassoXY[0] = _lassoXY[1] = 0;
 
-  addPointMode = lassoMode = selectionMode = false;
+  addPointMode = 0;
+  lassoMode = selectionMode = false;
   endSelection = undoSelection = invertSelection = quitSelection = 0;
+  changeSelection = 0;
 
   if(CTX::instance()->gamepad) Fl::add_timeout(.5, navigator_handler, (void*)this);
 }
@@ -218,7 +220,7 @@ void openglWindow::draw()
     glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
 
     _ctx->draw3d();
-    glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
+    glColor4ubv((GLubyte *) & CTX::instance()->color.geom.highlight[0]);
     float ps = CTX::instance()->geom.pointSize;
     if(_ctx->isHighResolution())
       ps *= CTX::instance()->highResolutionPointSizeFactor;
@@ -358,7 +360,7 @@ int openglWindow::handle(int event)
       std::vector<MElement*> elements;
       std::vector<SPoint2> points;
       std::vector<PView*> views;
-      _select(ENT_ALL, false, false, Fl::event_x(), Fl::event_y(), 5, 5,
+      _select(ENT_ALL, false, false, true, Fl::event_x(), Fl::event_y(), 5, 5,
               vertices, edges, faces, regions, elements, points, views);
       if(vertices.size() && CTX::instance()->geom.doubleClickedPointCommand.size()){
         CTX::instance()->geom.doubleClickedEntityTag = vertices[0]->tag();
@@ -638,7 +640,7 @@ int openglWindow::handle(int event)
           }
         }
       }
-      FlGui::instance()->elementaryContext->updatePoint(_point);
+      FlGui::instance()->elementaryContext->updatePoint(_point, addPointMode);
       redraw();
     }
     else{ // hover mode
@@ -651,6 +653,7 @@ int openglWindow::handle(int event)
         std::vector<SPoint2> points;
         std::vector<PView*> views;
         bool res = _select(_selection, false, CTX::instance()->mouseHoverMeshes,
+                           CTX::instance()->mouseHoverMeshes,
                            (int)_curr.win[0], (int)_curr.win[1], 5, 5,
                            vertices, edges, faces, regions, elements, points, views);
         if((_selection == ENT_ALL && res) ||
@@ -716,6 +719,8 @@ int openglWindow::pixel_w()
 {
 #if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3) && (FL_PATCH_VERSION >= 4)
   return Fl_Gl_Window::pixel_w();
+#elif (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION >= 4)
+  return Fl_Gl_Window::pixel_w();
 #else
   return w();
 #endif
@@ -725,12 +730,14 @@ int openglWindow::pixel_h()
 {
 #if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3) && (FL_PATCH_VERSION >= 4)
   return Fl_Gl_Window::pixel_h();
+#elif (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION >= 4)
+  return Fl_Gl_Window::pixel_h();
 #else
   return h();
 #endif
 }
 
-bool openglWindow::_select(int type, bool multiple, bool mesh,
+bool openglWindow::_select(int type, bool multiple, bool mesh, bool post,
                            int x, int y, int w, int h,
                            std::vector<GVertex*> &vertices,
                            std::vector<GEdge*> &edges,
@@ -746,7 +753,7 @@ bool openglWindow::_select(int type, bool multiple, bool mesh,
   if(_lock) return false;
   _lock = true;
   make_current();
-  bool ret = _ctx->select(type, multiple, mesh, x, y, w, h,
+  bool ret = _ctx->select(type, multiple, mesh, post, x, y, w, h,
                           vertices, edges, faces, regions, elements,
                           points, views);
   _lock = false;
@@ -768,6 +775,7 @@ char openglWindow::selectEntity(int type,
   _trySelection = 0;
   selectionMode = true;
   quitSelection = 0;
+  changeSelection = 0;
   endSelection = 0;
   undoSelection = 0;
   invertSelection = 0;
@@ -779,11 +787,16 @@ char openglWindow::selectEntity(int type,
     regions.clear();
     elements.clear();
     FlGui::instance()->wait();
+    if(changeSelection) {
+      Msg::Debug("Changing selection mode to %d", changeSelection);
+      _selection = changeSelection;
+      changeSelection = 0;
+    }
     if(quitSelection) {
       _selection = ENT_NONE;
       selectionMode = false;
       lassoMode = false;
-      addPointMode = false;
+      addPointMode = 0;
       cursor(FL_CURSOR_DEFAULT, FL_BLACK, FL_WHITE);
       return 'q';
     }
@@ -808,7 +821,7 @@ char openglWindow::selectEntity(int type,
         selectionMode = false;
         return 'c';
       }
-      else if(_select(_selection, multi, true, _trySelectionXYWH[0],
+      else if(_select(_selection, multi, true, false, _trySelectionXYWH[0],
                       _trySelectionXYWH[1], _trySelectionXYWH[2],
                       _trySelectionXYWH[3], vertices, edges, faces,
                       regions, elements, points, views)){
