@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -32,7 +32,7 @@ class openglWindow : public Fl_Gl_Window {
   double _lassoXY[2];
   void _drawScreenMessage();
   void _drawBorder();
-  bool _select(int type, bool multiple, bool mesh, int x, int y, int w, int h,
+  bool _select(int type, bool multiple, bool mesh, bool post, int x, int y, int w, int h,
                std::vector<GVertex*> &vertices, std::vector<GEdge*> &edges,
                std::vector<GFace*> &faces, std::vector<GRegion*> &regions,
                std::vector<MElement*> &elements, std::vector<SPoint2> &points,
@@ -45,8 +45,9 @@ class openglWindow : public Fl_Gl_Window {
   int pixel_h();
   time_t rawtime,  prev_rawtime;
   double response_frequency;
-  bool addPointMode, lassoMode, selectionMode;
-  int endSelection, undoSelection, invertSelection, quitSelection;
+  int addPointMode;
+  bool lassoMode, selectionMode;
+  int endSelection, undoSelection, invertSelection, quitSelection, changeSelection;
   std::string screenMessage[2];
   openglWindow(int x, int y, int w, int h);
   ~openglWindow();
@@ -62,6 +63,11 @@ class openglWindow : public Fl_Gl_Window {
   double frequency;
   void moveWithGamepad();
   Navigator *Nautilus;
+  void setPoint(double x, double y, double z)
+  {
+    _point[0] = x; _point[1] = y; _point[2] = z;
+  }
+
 };
 
 #endif
