@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -32,7 +32,14 @@ static void view_toggle_cb(Fl_Widget *w, void *data)
   int num = (intptr_t)data;
   viewButton *but = FlGui::instance()->onelab->getViewButton(num);
   if(but){
-    opt_view_visible(num, GMSH_SET, but->value());
+    if(Fl::event_state(FL_SHIFT)){
+      for(unsigned int i = 0; i < PView::list.size(); i++){
+        if(i != num) opt_view_visible(i, GMSH_SET | GMSH_GUI, 0);
+        else opt_view_visible(i, GMSH_SET | GMSH_GUI, 1);
+      }
+    }
+    else
+      opt_view_visible(num, GMSH_SET, but->value());
     drawContext::global()->draw();
   }
 }
