@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -25,7 +25,7 @@ class GEdgeCompound;
 class closestPointFinder;
 
 // A model edge.
-class GEdge : public GEntity{
+class GEdge : public GEntity {
  private:
   double _length;
   bool _tooSmall;
@@ -74,6 +74,9 @@ class GEdge : public GEntity{
   // set the visibility flag
   virtual void setVisibility(char val, bool recursive=false);
 
+  // set color
+  virtual void setColor(unsigned int val, bool recursive=false);
+
   // true if the edge is a seam for the given face.
   virtual bool isSeam(const GFace *face) const { return false; }
 
