@@ -135,6 +135,10 @@ class GModel {
   // entity that is currently being meshed (used for error reporting)
   GEntity *_currentMeshEntity;
 
+  // last entities/vertices where a meshing error has been reported
+  std::vector<GEntity*> _lastMeshEntityError;
+  std::vector<MVertex*> _lastMeshVertexError;
+
   // index of the current model (in the static list of all loaded
   // models)
   static int _current;
@@ -410,6 +414,14 @@ class GModel {
   void setCurrentMeshEntity(GEntity *e){ _currentMeshEntity = e; }
   GEntity *getCurrentMeshEntity(){ return _currentMeshEntity; }
 
+  // set/get entities/vertices linked meshing errors
+  void clearLastMeshEntityError(){ _lastMeshEntityError.clear(); }
+  void addLastMeshEntityError(GEntity *e){ _lastMeshEntityError.push_back(e); }
+  std::vector<GEntity*> getLastMeshEntityError(){ return _lastMeshEntityError; }
+  void clearLastMeshVertexError(){ _lastMeshVertexError.clear(); }
+  void addLastMeshVertexError(MVertex *v){ _lastMeshVertexError.push_back(v); }
+  std::vector<MVertex*> getLastMeshVertexError(){ return _lastMeshVertexError; }
+
   // delete all invisble mesh elements
   void removeInvisibleElements();
 
@@ -493,7 +505,7 @@ class GModel {
   // vertices that are too close, then merge edges, faces and
   // regions). Warning: the gluer changes the geometric model, so that
   // some pointers could become invalid.
-  void glue(double eps);
+  //  void glue(double eps);
 
   // change the entity creation factory
   void setFactory(std::string name);
@@ -724,6 +736,9 @@ class GModel {
 
   // SU2 mesh file
   int writeSU2(const std::string &name, bool saveAll, double scalingFactor);
+
+  // GAMBIT neutral mesh file (.neu)
+  int writeNEU(const std::string &name, bool saveAll, double scalingFactor);
 };
 
 #endif
