@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -17,7 +17,9 @@
 #include "GModel.h"
 #include "Numeric.h"
 #include "ExtrudeParams.h"
+#include "GModelIO_GEO.h"
 #include "Geo.h"
+#include "Parser.h"
 #include "GmshDefines.h"
 
 GVertex *GeoFactory::addVertex(GModel *gm, double x, double y, double z, double lc)
@@ -30,7 +32,7 @@ GVertex *GeoFactory::addVertex(GModel *gm, double x, double y, double z, double
   lc *= CTX::instance()->geom.scalingFactor;
   if(lc == 0.) lc = MAX_LC; // no mesh size given at the point
   Vertex *p;
-  p = Create_Vertex(num, x, y, z, lc, 1.0);
+  p = CreateVertex(num, x, y, z, lc, 1.0);
   Tree_Add(gm->getGEOInternals()->Points, &p);
   p->Typ = MSH_POINT;
   p->Num = num;
@@ -50,8 +52,8 @@ GEdge *GeoFactory::addLine(GModel *gm, GVertex *start, GVertex *end)
   List_Add(iList, &tagBeg);
   List_Add(iList, &tagEnd);
 
-  Curve *c = Create_Curve(num, MSH_SEGM_LINE, 1, iList, NULL,
-			  -1, -1, 0., 1.);
+  Curve *c = CreateCurve(num, MSH_SEGM_LINE, 1, iList, NULL,
+        -1, -1, 0., 1.);
   Tree_Add(gm->getGEOInternals()->Curves, &c);
   CreateReversedCurve(c);
   List_Delete(iList);
@@ -104,7 +106,7 @@ GRegion* GeoFactory::addVolume (GModel *gm, std::vector<std::vector<GFace *> > f
       numfl++;
       if (!FindSurfaceLoop(numfl)) break;
     }
-    SurfaceLoop *l = Create_SurfaceLoop(numfl, temp);
+    SurfaceLoop *l = CreateSurfaceLoop(numfl, temp);
     vecLoops.push_back(l);
     Tree_Add(gm->getGEOInternals()->SurfaceLoops, &l);
     List_Delete(temp);
@@ -112,13 +114,13 @@ GRegion* GeoFactory::addVolume (GModel *gm, std::vector<std::vector<GFace *> > f
 
   //create volume
   int numv = gm->getMaxElementaryNumber(3) + 1;
-  Volume *v = Create_Volume(numv, MSH_VOLUME);
+  Volume *v = CreateVolume(numv, MSH_VOLUME);
   List_T *temp = List_Create(nLoops, nLoops, sizeof(int));
   for (unsigned int i = 0; i < vecLoops.size(); i++){
     int numl = vecLoops[i]->Num;
     List_Add(temp, &numl);
   }
-  setVolumeSurfaces(v, temp);
+  SetVolumeSurfaces(v, temp);
   List_Delete(temp);
   Tree_Add(gm->getGEOInternals()->Volumes, &v);
   v->Typ = MSH_VOLUME;
@@ -143,8 +145,8 @@ GEdge* GeoFactory::addCircleArc(GModel *gm,GVertex *begin, GVertex *center, GVer
   List_Add(iList, &tagMiddle);
   List_Add(iList, &tagEnd);
 
-  Curve *c = Create_Curve(num, MSH_SEGM_CIRC, 1, iList, NULL,
-			  -1, -1, 0., 1.);
+  Curve *c = CreateCurve(num, MSH_SEGM_CIRC, 1, iList, NULL,
+        -1, -1, 0., 1.);
   Tree_Add(gm->getGEOInternals()->Curves, &c);
   CreateReversedCurve(c);
   List_Delete(iList);
@@ -175,28 +177,24 @@ std::vector<GFace *> GeoFactory::addRuledFaces(GModel *gm,
       int numEdge = edges[i][j]->tag();
       List_Add(temp, &numEdge);
     }
-    int type=ENT_LINE;
-    if(select_contour(type, edges[i][0]->tag(), temp))
-    {
-        sortEdgesInLoop(numl, temp);
-        EdgeLoop *l = Create_EdgeLoop(numl, temp);
-        vecLoops.push_back(l);
-        Tree_Add(gm->getGEOInternals()->EdgeLoops, &l);
-        l->Num = numl;
-    }
+    SortEdgesInLoop(numl, temp);
+    EdgeLoop *l = CreateEdgeLoop(numl, temp);
+    vecLoops.push_back(l);
+    Tree_Add(gm->getGEOInternals()->EdgeLoops, &l);
+    l->Num = numl;
     List_Delete(temp);
   }
 
   //create plane surfaces
   int numf = gm->getMaxElementaryNumber(2) + 1;
-  Surface *s = Create_Surface(numf, MSH_SURF_TRIC);
+  Surface *s = CreateSurface(numf, MSH_SURF_TRIC);
   List_T *iList = List_Create(nLoops, nLoops, sizeof(int));
   for (unsigned int i=0; i< vecLoops.size(); i++){
     int numl = vecLoops[i]->Num;
     List_Add(iList, &numl);
   }
-  setSurfaceGeneratrices(s, iList);
-  End_Surface(s);
+  SetSurfaceGeneratrices(s, iList);
+  EndSurface(s);
   Tree_Add(gm->getGEOInternals()->Surfaces, &s);
   s->Typ= MSH_SURF_TRIC;
   s->Num = numf;
@@ -278,7 +276,7 @@ std::vector<GEntity*> GeoFactory::extrudeBoundaryLayer(GModel *gm, GEntity *e,
                 list_out);
 
   //create GEntities
-  gm->importGEOInternals();
+  gm->getGEOInternals()->synchronize(gm);
 
   //return the new created entity
   int nbout = List_Nbr(list_out);
@@ -329,8 +327,8 @@ std::vector<GEntity*> GeoFactory::extrudeBoundaryLayer(GModel *gm, GEntity *e,
   //      ge->geomType() == GEntity::BoundaryLayerCurve){
   //     ExtrudeParams *ep = ge->meshAttributes.extrude;
   //     if(ep && ep->mesh.ExtrudeMesh && ep->geo.Mode == COPIED_ENTITY &&
-  // 	std::abs(ep->geo.Source) ==e->tag() )
-  // 	  newEnt = ge;
+  //  std::abs(ep->geo.Source) ==e->tag() )
+  //    newEnt = ge;
   //     }
   //   }
   // }
@@ -338,11 +336,11 @@ std::vector<GEntity*> GeoFactory::extrudeBoundaryLayer(GModel *gm, GEntity *e,
   //   for(GModel::fiter it = gm->firstFace(); it != gm->lastFace(); it++){
   //     GFace *gf = *it;
   //     if(gf->getNativeType() == GEntity::GmshModel &&
-  // 	 gf->geomType() == GEntity::BoundaryLayerSurface){
-  // 	ExtrudeParams *ep = gf->meshAttributes.extrude;
-  // 	if(ep && ep->mesh.ExtrudeMesh && ep->geo.Mode == COPIED_ENTITY
-  // 	   && std::abs(ep->geo.Source) == e->tag())
-  // 	  newEnt = gf;
+  //   gf->geomType() == GEntity::BoundaryLayerSurface){
+  //  ExtrudeParams *ep = gf->meshAttributes.extrude;
+  //  if(ep && ep->mesh.ExtrudeMesh && ep->geo.Mode == COPIED_ENTITY
+  //     && std::abs(ep->geo.Source) == e->tag())
+  //    newEnt = gf;
   //     }
   //   }
   // }
@@ -357,7 +355,7 @@ void GeoFactory::healGeometry(GModel *gm, double tolerance)
   GModel::setCurrent(gm);
   ReplaceAllDuplicatesNew(tolerance);
   gm->destroy();
-  gm->importGEOInternals();
+  gm->getGEOInternals()->synchronize(gm);
   GModel::setCurrent(current);
 }
 
@@ -377,50 +375,50 @@ GFace *GeoFactory::_addPlanarFace(GModel *gm, const std::vector<std::vector<GEdg
       //create curve if it does not exist
       Curve *c = FindCurve(numEdge);
       if(!c){
-	GVertex *gvb = ge->getBeginVertex();
-	GVertex *gve = ge->getEndVertex();
-	Vertex *vertb = FindPoint(abs(gvb->tag()));
-	Vertex *verte = FindPoint(abs(gve->tag()));
-	if (!vertb){
-	  vertb = Create_Vertex(gvb->tag(), gvb->x(), gvb->y(), gvb->z(),
-				gvb->prescribedMeshSizeAtVertex(), 1.0);
-	  Tree_Add(gm->getGEOInternals()->Points, &vertb);
-	  vertb->Typ = MSH_POINT;
-	  vertb->Num = gvb->tag();
-	 }
-	if (!verte){
-	  verte = Create_Vertex(gve->tag(), gve->x(), gve->y(), gve->z(),
-				gve->prescribedMeshSizeAtVertex(), 1.0);
-	  Tree_Add(gm->getGEOInternals()->Points, &verte);
-	  verte->Typ = MSH_POINT;
-	  verte->Num = gve->tag();
-	}
+  GVertex *gvb = ge->getBeginVertex();
+  GVertex *gve = ge->getEndVertex();
+  Vertex *vertb = FindPoint(abs(gvb->tag()));
+  Vertex *verte = FindPoint(abs(gve->tag()));
+  if (!vertb){
+    vertb = CreateVertex(gvb->tag(), gvb->x(), gvb->y(), gvb->z(),
+        gvb->prescribedMeshSizeAtVertex(), 1.0);
+    Tree_Add(gm->getGEOInternals()->Points, &vertb);
+    vertb->Typ = MSH_POINT;
+    vertb->Num = gvb->tag();
+   }
+  if (!verte){
+    verte = CreateVertex(gve->tag(), gve->x(), gve->y(), gve->z(),
+        gve->prescribedMeshSizeAtVertex(), 1.0);
+    Tree_Add(gm->getGEOInternals()->Points, &verte);
+    verte->Typ = MSH_POINT;
+    verte->Num = gve->tag();
+  }
 
-	if (ge->geomType() == GEntity::Line){
-	  c = Create_Curve(numEdge, MSH_SEGM_LINE, 1, NULL, NULL, -1, -1, 0., 1.);
-	}
-	else if (ge->geomType() == GEntity::DiscreteCurve){
-	  c = Create_Curve(numEdge, MSH_SEGM_DISCRETE, 1, NULL, NULL, -1, -1, 0., 1.);
-	}
-	else if(ge->geomType() == GEntity::CompoundCurve){
-	  c = Create_Curve(numEdge, MSH_SEGM_COMPOUND, 1, NULL, NULL, -1, -1, 0., 1.);
-	  std::vector<GEdge*> gec = ((GEdgeCompound*)ge)->getCompounds();
-	  for(unsigned int i = 0; i < gec.size(); i++)
-	    c->compound.push_back(gec[i]->tag());
-	}
-	else{
-	  c = Create_Curve(numEdge, MSH_SEGM_DISCRETE, 1, NULL, NULL, -1, -1, 0., 1.);
-	}
+  if (ge->geomType() == GEntity::Line){
+    c = CreateCurve(numEdge, MSH_SEGM_LINE, 1, NULL, NULL, -1, -1, 0., 1.);
+  }
+  else if (ge->geomType() == GEntity::DiscreteCurve){
+    c = CreateCurve(numEdge, MSH_SEGM_DISCRETE, 1, NULL, NULL, -1, -1, 0., 1.);
+  }
+  else if(ge->geomType() == GEntity::CompoundCurve){
+    c = CreateCurve(numEdge, MSH_SEGM_COMPOUND, 1, NULL, NULL, -1, -1, 0., 1.);
+    std::vector<GEdge*> gec = ((GEdgeCompound*)ge)->getCompounds();
+    for(unsigned int i = 0; i < gec.size(); i++)
+      c->compound.push_back(gec[i]->tag());
+  }
+  else{
+    c = CreateCurve(numEdge, MSH_SEGM_DISCRETE, 1, NULL, NULL, -1, -1, 0., 1.);
+  }
 
-	c->Control_Points = List_Create(2, 1, sizeof(Vertex *));
-	List_Add(c->Control_Points, &vertb);
-	List_Add(c->Control_Points, &verte);
-	c->beg = vertb;
-	c->end = verte;
-	End_Curve(c);
+  c->Control_Points = List_Create(2, 1, sizeof(Vertex *));
+  List_Add(c->Control_Points, &vertb);
+  List_Add(c->Control_Points, &verte);
+  c->beg = vertb;
+  c->end = verte;
+  EndCurve(c);
 
-	Tree_Add(gm->getGEOInternals()->Curves, &c);
-	CreateReversedCurve(c);
+  Tree_Add(gm->getGEOInternals()->Curves, &c);
+  CreateReversedCurve(c);
       }
       int signedNumEdge = numEdge*signedEdge.getSign();
       List_Add(temp, &signedNumEdge);
@@ -431,8 +429,8 @@ GFace *GeoFactory::_addPlanarFace(GModel *gm, const std::vector<std::vector<GEdg
       numl++;
       if (!FindEdgeLoop(numl)) break;
     }
-    sortEdgesInLoop(numl, temp, orientEdges);
-    EdgeLoop *l = Create_EdgeLoop(numl, temp);
+    SortEdgesInLoop(numl, temp, orientEdges);
+    EdgeLoop *l = CreateEdgeLoop(numl, temp);
     vecLoops.push_back(l);
     Tree_Add(gm->getGEOInternals()->EdgeLoops, &l);
     l->Num = numl;
@@ -441,16 +439,16 @@ GFace *GeoFactory::_addPlanarFace(GModel *gm, const std::vector<std::vector<GEdg
 
   //create surface
   int numf  = gm->getMaxElementaryNumber(2)+1;
-  Surface *s = Create_Surface(numf, MSH_SURF_PLAN);
+  Surface *s = CreateSurface(numf, MSH_SURF_PLAN);
   List_T *temp = List_Create(nLoops, nLoops, sizeof(int));
   for (int i = 0; i < nLoops; i++){
     int numl = vecLoops[i]->Num;
     List_Add(temp, &numl);
   }
 
-  setSurfaceGeneratrices(s, temp);
+  SetSurfaceGeneratrices(s, temp);
   List_Delete(temp);
-  End_Surface(s);
+  EndSurface(s);
   Tree_Add(gm->getGEOInternals()->Surfaces, &s);
 
   //gmsh surface
@@ -461,68 +459,123 @@ GFace *GeoFactory::_addPlanarFace(GModel *gm, const std::vector<std::vector<GEdg
 }
 
 #if defined(HAVE_OCC)
-#include "OCCIncludes.h"
 #include "GModelIO_OCC.h"
 #include "OCCVertex.h"
 #include "OCCEdge.h"
 #include "OCCFace.h"
 #include "OCCRegion.h"
-#include "BRepBuilderAPI_MakeVertex.hxx"
-#include "BRepOffsetAPI_MakePipe.hxx"
-#include "BRepBuilderAPI_MakeEdge.hxx"
-#include "BRepPrimAPI_MakeRevol.hxx"
-#include "BRepPrimAPI_MakePrism.hxx"
-#include "BRepBuilderAPI_MakeWire.hxx"
-#include "BRepOffsetAPI_ThruSections.hxx"
-#include "BRepOffsetAPI_MakeFilling.hxx"
-#include "BRepBuilderAPI_MakeFace.hxx"
+
+#include <BRepAdaptor_Curve.hxx>
+#include <BRepAdaptor_Surface.hxx>
+#include <BRepAlgoAPI_Common.hxx>
+#include <BRepAlgoAPI_Cut.hxx>
+#include <BRepAlgoAPI_Fuse.hxx>
+#include <BRepAlgoAPI_Section.hxx>
+#include <BRepBndLib.hxx>
+#include <BRepBuilderAPI_MakeEdge.hxx>
+#include <BRepBuilderAPI_MakeFace.hxx>
+#include <BRepBuilderAPI_MakeShell.hxx>
+#include <BRepBuilderAPI_MakeSolid.hxx>
+#include <BRepBuilderAPI_MakeVertex.hxx>
+#include <BRepBuilderAPI_MakeWire.hxx>
+#include <BRepBuilderAPI_Transform.hxx>
+#include <BRepBuilderAPI_GTransform.hxx>
+#include <BRepCheck_Analyzer.hxx>
+#include <BRepFilletAPI_MakeFillet.hxx>
+#include <BRepGProp.hxx>
+#include <BRepLProp_SLProps.hxx>
+#include <BRepLib.hxx>
+#include <BRepMesh.hxx>
+#include <BRepMesh_IncrementalMesh.hxx>
+#include <BRepOffsetAPI_MakeFilling.hxx>
+#include <BRepOffsetAPI_MakePipe.hxx>
+#include <BRepOffsetAPI_Sewing.hxx>
+#include <BRepOffsetAPI_ThruSections.hxx>
+#include <BRepPrimAPI_MakeBox.hxx>
+#include <BRepPrimAPI_MakeCone.hxx>
+#include <BRepPrimAPI_MakeCylinder.hxx>
+#include <BRepPrimAPI_MakePrism.hxx>
+#include <BRepPrimAPI_MakeRevol.hxx>
+#include <BRepPrimAPI_MakeSphere.hxx>
+#include <BRepPrimAPI_MakeTorus.hxx>
+#include <BRepTools.hxx>
+#include <BRepTools_WireExplorer.hxx>
+#include <BRep_Builder.hxx>
+#include <BRep_Tool.hxx>
+#include <Bnd_Box.hxx>
+#include <ElCLib.hxx>
 #include <GC_MakeArcOfCircle.hxx>
-#include <gp_Pnt.hxx>
-#include <gp_Vec.hxx>
+#include <GProp_GProps.hxx>
+#include <Geom2d_Curve.hxx>
+#include <GeomAPI_ExtremaCurveCurve.hxx>
+#include <GeomAPI_PointsToBSpline.hxx>
+#include <GeomAPI_ProjectPointOnCurve.hxx>
+#include <GeomAPI_ProjectPointOnSurf.hxx>
+#include <GeomLProp_SLProps.hxx>
+#include <Geom_BSplineCurve.hxx>
+#include <Geom_BezierCurve.hxx>
+#include <Geom_Circle.hxx>
+#include <Geom_Curve.hxx>
+#include <Geom_Surface.hxx>
+#include <Geom_TrimmedCurve.hxx>
+#include <IGESControl_Reader.hxx>
+#include <IGESControl_Writer.hxx>
+#include <IGESToBRep_Reader.hxx>
+#include <Interface_Static.hxx>
+#include <Poly_Array1OfTriangle.hxx>
+#include <Poly_Triangle.hxx>
+#include <Poly_Triangulation.hxx>
+#include <Precision.hxx>
+#include <STEPControl_Reader.hxx>
+#include <STEPControl_StepModelType.hxx>
+#include <STEPControl_Writer.hxx>
+#include <ShapeAnalysis.hxx>
+#include <ShapeAnalysis_CheckSmallFace.hxx>
+#include <ShapeAnalysis_DataMapOfShapeListOfReal.hxx>
+#include <ShapeAnalysis_ShapeContents.hxx>
+#include <ShapeAnalysis_ShapeTolerance.hxx>
+#include <ShapeBuild_ReShape.hxx>
+#include <ShapeFix.hxx>
+#include <ShapeFix_FixSmallFace.hxx>
+#include <ShapeFix_Shape.hxx>
+#include <ShapeFix_Wireframe.hxx>
+#include <ShapeUpgrade_ShellSewing.hxx>
+#include <Standard_ErrorHandler.hxx>
+#include <Standard_Failure.hxx>
+#include <Standard_Version.hxx>
+#include <StlAPI_Writer.hxx>
+#include <TColStd_HArray1OfInteger.hxx>
+#include <TColStd_HArray1OfReal.hxx>
+#include <TColgp_Array1OfPnt2d.hxx>
+#include <TColgp_HArray1OfPnt.hxx>
+#include <TopExp.hxx>
+#include <TopExp_Explorer.hxx>
+#include <TopTools_DataMapOfIntegerShape.hxx>
+#include <TopTools_DataMapOfShapeInteger.hxx>
+#include <TopTools_IndexedMapOfShape.hxx>
+#include <TopTools_ListIteratorOfListOfShape.hxx>
+#include <TopTools_ListOfShape.hxx>
+#include <TopoDS.hxx>
+#include <TopoDS_Compound.hxx>
+#include <TopoDS_Edge.hxx>
+#include <TopoDS_Face.hxx>
+#include <TopoDS_Iterator.hxx>
+#include <TopoDS_Shape.hxx>
+#include <TopoDS_Solid.hxx>
+#include <TopoDS_Wire.hxx>
 #include <gce_MakeCirc.hxx>
 #include <gce_MakePln.hxx>
-#include <ElCLib.hxx>
-#include <Geom_Circle.hxx>
-#include <Geom_TrimmedCurve.hxx>
-#include <Geom_BezierCurve.hxx>
-#include <Geom_BSplineCurve.hxx>
-#include <BRepBuilderAPI_Transform.hxx>
-#include <TColgp_HArray1OfPnt.hxx>
-#include <TColStd_HArray1OfReal.hxx>
-#include <TColStd_HArray1OfInteger.hxx>
-#include <BRep_Builder.hxx>
-#include <BRepPrimAPI_MakeBox.hxx>
-#include <BRepPrimAPI_MakeTorus.hxx>
-#include <BRepPrimAPI_MakeSphere.hxx>
-#include <BRepPrimAPI_MakeCylinder.hxx>
-#include <GeomLProp_SLProps.hxx>
-
-#include <BRepAlgoAPI_Section.hxx>
-#include <GProp_GProps.hxx>
-#include <BRepGProp.hxx>
-#include <TopoDS_Shape.hxx>
-#include <BRepTools.hxx>
-#include <BRepBndLib.hxx>
 #include <gp.hxx>
 #include <gp_Ax1.hxx>
 #include <gp_Ax2.hxx>
 #include <gp_Ax2d.hxx>
 #include <gp_Dir.hxx>
 #include <gp_Dir2d.hxx>
+#include <gp_Pnt.hxx>
 #include <gp_Pnt2d.hxx>
 #include <gp_Trsf.hxx>
-#include <TopExp_Explorer.hxx>
-#include <TopoDS.hxx>
-#include <TopoDS_Edge.hxx>
-#include <TopoDS_Face.hxx>
-#include <TopoDS_Wire.hxx>
-#include <TopoDS_Compound.hxx>
-#include <TopTools_ListOfShape.hxx>
-#include <GeomAPI_PointsToBSpline.hxx>
-#include "OCC_Connect.h"
-#if defined(HAVE_SALOME)
-#include "Partition_Spliter.hxx"
-#endif
+#include <gp_GTrsf.hxx>
+#include <gp_Vec.hxx>
 
 GVertex *OCCFactory::addVertex(GModel *gm, double x, double y, double z, double lc)
 {
@@ -553,7 +606,7 @@ GEdge *OCCFactory::addLine(GModel *gm, GVertex *start, GVertex *end)
   TopoDS_Edge occEdge;
   if (occv1 && occv2){
      occEdge = BRepBuilderAPI_MakeEdge(occv1->getShape(),
-				       occv2->getShape()).Edge();
+               occv2->getShape()).Edge();
   }
   else{
     gp_Pnt p1(start->x(),start->y(),start->z());
@@ -605,7 +658,7 @@ GEdge *OCCFactory::addCircleArc(GModel *gm, const arcCreationMethod &method,
 
 GEdge *OCCFactory::addSpline(GModel *gm, const splineType &type,
                              GVertex *start, GVertex *end,
-			     std::vector<std::vector<double> > points)
+           std::vector<std::vector<double> > points)
 {
   if (!gm->_occ_internals)
     gm->_occ_internals = new OCC_Internals;
@@ -651,10 +704,10 @@ GEdge *OCCFactory::addSpline(GModel *gm, const splineType &type,
 
 
 GEdge *OCCFactory::addNURBS(GModel *gm, GVertex *start, GVertex *end,
-			    std::vector<std::vector<double> > points,
-			    std::vector<double> knots,
-			    std::vector<double> weights,
-			    std::vector<int> mult)
+          std::vector<std::vector<double> > points,
+          std::vector<double> knots,
+          std::vector<double> weights,
+          std::vector<int> mult)
 {
   try{
   if (!gm->_occ_internals)
@@ -684,7 +737,7 @@ GEdge *OCCFactory::addNURBS(GModel *gm, GVertex *start, GVertex *end,
 
   const int degree = totKnots - nbControlPoints - 1;
   Msg::Debug("creation of a nurbs of degree %d with %d control points",
-	     degree,nbControlPoints);
+       degree,nbControlPoints);
 
   int index = 1;
   ctrlPoints.SetValue(index++, gp_Pnt(start->x(), start->y(), start->z()));
@@ -712,25 +765,25 @@ GEdge *OCCFactory::addNURBS(GModel *gm, GVertex *start, GVertex *end,
 /* add2Drect: rectangluar face, given lower left point and width/height (in X-Y plane)*/
 GFace *OCCFactory::add2Drect(GModel *gm,double x0, double y0, double dx, double dy)
 {
-	Msg::Info("Default working plane is XY in add2D* functions...");
+  Msg::Info("Default working plane is XY in add2D* functions...");
 
-	if (!gm->_occ_internals)
-		gm->_occ_internals = new OCC_Internals;
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
 
-	TopoDS_Vertex occv1 = BRepBuilderAPI_MakeVertex( gp_Pnt(x0   , y0   , 0.) );
-	TopoDS_Vertex occv2 = BRepBuilderAPI_MakeVertex( gp_Pnt(x0+dx, y0   , 0.) );
-	TopoDS_Vertex occv3 = BRepBuilderAPI_MakeVertex( gp_Pnt(x0+dx, y0+dy, 0.) );
-	TopoDS_Vertex occv4 = BRepBuilderAPI_MakeVertex( gp_Pnt(x0   , y0+dy, 0.) );
+  TopoDS_Vertex occv1 = BRepBuilderAPI_MakeVertex( gp_Pnt(x0   , y0   , 0.) );
+  TopoDS_Vertex occv2 = BRepBuilderAPI_MakeVertex( gp_Pnt(x0+dx, y0   , 0.) );
+  TopoDS_Vertex occv3 = BRepBuilderAPI_MakeVertex( gp_Pnt(x0+dx, y0+dy, 0.) );
+  TopoDS_Vertex occv4 = BRepBuilderAPI_MakeVertex( gp_Pnt(x0   , y0+dy, 0.) );
 
-	TopoDS_Edge occEdge1 = BRepBuilderAPI_MakeEdge(occv1,occv2);
-	TopoDS_Edge occEdge2 = BRepBuilderAPI_MakeEdge(occv2,occv3);
-	TopoDS_Edge occEdge3 = BRepBuilderAPI_MakeEdge(occv3,occv4);
-	TopoDS_Edge occEdge4 = BRepBuilderAPI_MakeEdge(occv4,occv1);
+  TopoDS_Edge occEdge1 = BRepBuilderAPI_MakeEdge(occv1,occv2);
+  TopoDS_Edge occEdge2 = BRepBuilderAPI_MakeEdge(occv2,occv3);
+  TopoDS_Edge occEdge3 = BRepBuilderAPI_MakeEdge(occv3,occv4);
+  TopoDS_Edge occEdge4 = BRepBuilderAPI_MakeEdge(occv4,occv1);
 
-	TopoDS_Wire rectWire = BRepBuilderAPI_MakeWire(occEdge1 , occEdge2 , occEdge3, occEdge4);
-	TopoDS_Face rectFace = BRepBuilderAPI_MakeFace(rectWire);
+  TopoDS_Wire rectWire = BRepBuilderAPI_MakeWire(occEdge1 , occEdge2 , occEdge3, occEdge4);
+  TopoDS_Face rectFace = BRepBuilderAPI_MakeFace(rectWire);
 
-	return gm->_occ_internals->addFaceToModel(gm, TopoDS::Face(rectFace));
+  return gm->_occ_internals->addFaceToModel(gm, TopoDS::Face(rectFace));
 }
 
 /* add2Dellips: ellips face, given lower left point and width/height (in X-Y plane)*/
@@ -756,38 +809,6 @@ GFace *OCCFactory::add2Dellips(GModel *gm, double xc, double yc, double rx, doub
   return gm->_occ_internals->addFaceToModel(gm, TopoDS::Face(ellipsFace));
 }
 
-/*
-GEdge *OCCFactory::addBezierSurface(GModel *gm,
-				    std::vector<GEdge *> & wires, // four edges indeed
-				    std::vector<std::vector<double> > points)
-{
-
-  TColgp_Array2OfPnt ctrlPoints(1, nbControlPoints + 2);
-  int index = 1;
-  ctrlPoints.SetValue(index++, gp_Pnt(start->x(), start->y(), start->z()));
-  for (int i = 0; i < nbControlPoints; i++) {
-    gp_Pnt aP(points[i][0],points[i][1],points[i][2]);
-    ctrlPoints.SetValue(index++, aP);
-  }
-
-
-  BRepBuilderAPI_MakeFace aGenerator (aBezierSurface);
-  BRepBuilderAPI_MakeWire wire_maker;
-  for (unsigned j = 0; j < wires.size(); j++) {
-    GEdge *ge = wires[j];
-    OCCEdge *occe = dynamic_cast<OCCEdge*>(ge);
-    if (occe){
-      wire_maker.Add(occe->getTopoDS_Edge());
-    }
-  }
-  TopoDS_Wire myWire = wire_maker.Wire();
-  aGenerator.Add (myWire);
-  aGenerator.Build();
-  TopoDS_Shape aResult = aGenerator.Shape();
-  return gm->_occ_internals->addFaceToModel(gm, TopoDS::Face(aResult));
-}
-*/
-
 GEntity *OCCFactory::revolve(GModel *gm, GEntity* base,
                              std::vector<double> p1,
                              std::vector<double> p2, double angle)
@@ -865,7 +886,7 @@ GEntity *OCCFactory::addSphere(GModel *gm, double xc, double yc, double zc, doub
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return gm->_occ_internals->getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
+  return gm->_occ_internals->getRegionForOCCShape(gm, TopoDS::Solid(shape));
 }
 
 GRegion* OCCFactory::addVolume (GModel *gm, std::vector<std::vector<GFace *> > faces)
@@ -903,7 +924,7 @@ GEntity *OCCFactory::addCylinder(GModel *gm, std::vector<double> p1,
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return gm->_occ_internals->getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
+  return gm->_occ_internals->getRegionForOCCShape(gm, TopoDS::Solid(shape));
 }
 
 GEntity *OCCFactory::addTorus(GModel *gm, std::vector<double> p1,
@@ -935,7 +956,7 @@ GEntity *OCCFactory::addTorus(GModel *gm, std::vector<double> p1,
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return gm->_occ_internals->getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
+  return gm->_occ_internals->getRegionForOCCShape(gm, TopoDS::Solid(shape));
 }
 
 GEntity *OCCFactory::addCone(GModel *gm,  std::vector<double> p1,
@@ -968,7 +989,7 @@ GEntity *OCCFactory::addCone(GModel *gm,  std::vector<double> p1,
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return gm->_occ_internals->getOCCRegionByNativePtr(gm,TopoDS::Solid(shape));
+  return gm->_occ_internals->getRegionForOCCShape(gm,TopoDS::Solid(shape));
 }
 
 GEntity *OCCFactory::addBlock(GModel *gm, std::vector<double> p1,
@@ -990,7 +1011,7 @@ GEntity *OCCFactory::addBlock(GModel *gm, std::vector<double> p1,
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return gm->_occ_internals->getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
+  return gm->_occ_internals->getRegionForOCCShape(gm, TopoDS::Solid(shape));
 }
 
 GEntity *OCCFactory::add3DBlock(GModel *gm,std::vector<double> p1,
@@ -1011,7 +1032,7 @@ GEntity *OCCFactory::add3DBlock(GModel *gm,std::vector<double> p1,
   gm->destroy();
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
-  return gm->_occ_internals->getOCCRegionByNativePtr(gm, TopoDS::Solid(shape));
+  return gm->_occ_internals->getRegionForOCCShape(gm, TopoDS::Solid(shape));
 }
 
 GModel *OCCFactory::computeBooleanUnion(GModel* obj, GModel* tool,
@@ -1026,11 +1047,11 @@ GModel *OCCFactory::computeBooleanUnion(GModel* obj, GModel* tool,
     if (createNewModel){
       GModel *temp = new GModel;
       temp->_occ_internals = new OCC_Internals;
-      temp->_occ_internals->addShapeToLists(occ_obj->getShape());
+      temp->_occ_internals->_addShapeToLists(occ_obj->getShape());
       obj = temp;
     }
     obj->_occ_internals->applyBooleanOperator(occ_tool->getShape(),
-                                              OCC_Internals::Fuse);
+                                              OCC_Internals::Union);
     obj->destroy();
     obj->_occ_internals->buildLists();
     obj->_occ_internals->buildGModel(obj);
@@ -1054,11 +1075,11 @@ GModel *OCCFactory::computeBooleanDifference(GModel* obj, GModel* tool,
     if (createNewModel){
       GModel *temp = new GModel;
       temp->_occ_internals = new OCC_Internals;
-      temp->_occ_internals->addShapeToLists(occ_obj->getShape());
+      temp->_occ_internals->_addShapeToLists(occ_obj->getShape());
       obj = temp;
     }
     obj->getOCCInternals()->applyBooleanOperator(occ_tool->getShape(),
-                                                 OCC_Internals::Cut);
+                                                 OCC_Internals::Difference);
     obj->destroy();
     obj->_occ_internals->buildLists();
     obj->_occ_internals->buildGModel(obj);
@@ -1081,7 +1102,7 @@ GModel *OCCFactory::computeBooleanIntersection(GModel* obj, GModel* tool,
     if (createNewModel){
       GModel *temp = new GModel;
       temp->_occ_internals = new OCC_Internals;
-      temp->_occ_internals->addShapeToLists(occ_obj->getShape());
+      temp->_occ_internals->_addShapeToLists(occ_obj->getShape());
       obj = temp;
     }
     obj->getOCCInternals()->applyBooleanOperator(occ_tool->getShape(),
@@ -1096,48 +1117,6 @@ GModel *OCCFactory::computeBooleanIntersection(GModel* obj, GModel* tool,
   return obj;
 }
 
-/* same as checkbox GUI - works a bit better than occconnect... */
-void OCCFactory::salomeconnect(GModel *gm)
-{
-#if defined(HAVE_SALOME)
-  Msg::Info("- cutting and connecting faces with Salome's Partition_Spliter");
-  TopExp_Explorer e2;
-  Partition_Spliter ps;
-  TopoDS_Shape shape = gm->_occ_internals->getShape();
-  for (e2.Init(shape, TopAbs_SOLID); e2.More(); e2.Next())
-    ps.AddShape(e2.Current());
-  try{
-    ps.Compute();
-    shape = ps.Shape();
-    gm->destroy();
-    gm->_occ_internals->loadShape(&shape);
-    gm->_occ_internals->buildLists();
-    gm->_occ_internals->buildGModel(gm);
-  }
-  catch(Standard_Failure &err){
-    Msg::Error("%s", err.GetMessageString());
-  }
-#else
-  Msg::Info("You need to recompile with Salome support to use Salome's Partition_Spliter");
-#endif
-}
-
-/* same as checkbox GUI - does not work at all, though!*/
-void OCCFactory::occconnect(GModel *gm)
-{
-  Msg::Info("- cutting and connecting faces with OCC_Connect");
-  OCC_Connect connect(1);
-  TopoDS_Shape shape = gm->_occ_internals->getShape();
-  for(TopExp_Explorer p(shape, TopAbs_SOLID); p.More(); p.Next())
-    connect.Add(p.Current());
-  connect.Connect();
-  shape = connect;
-  gm->destroy();
-  gm->_occ_internals->loadShape(&shape);
-  gm->_occ_internals->buildLists();
-  gm->_occ_internals->buildGModel(gm);
-}
-
 /* IsEqualG : a tolerance function for setPeriodicAllFaces */
 bool IsEqualG(double x, double y)
 {
@@ -1149,6 +1128,9 @@ bool IsEqualG(double x, double y)
    setPeriodicPairOfFaces to make them all periodic */
 void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTranslationVector)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
   Msg::Info("Experimental: search for 'translated' faces and calls setPeriodicPairOfFaces automatically");
   TopoDS_Shape shape = gm->_occ_internals->getShape();
   gp_Trsf theTransformation;
@@ -1192,8 +1174,8 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
          && IsEqualG(BarycenterFace1.X()+FaceTranslationVector[0],BarycenterFace2.X())
          && IsEqualG(BarycenterFace1.Y()+FaceTranslationVector[1],BarycenterFace2.Y())
          && IsEqualG(BarycenterFace1.Z()+FaceTranslationVector[2],BarycenterFace2.Z())){
-        numFaceMaster = gm->getOCCInternals()->getOCCFaceByNativePtr(gm,aFace1)->tag();
-        numFaceSlave  = gm->getOCCInternals()->getOCCFaceByNativePtr(gm,aFace2)->tag();
+        numFaceMaster = gm->getOCCInternals()->getFaceForOCCShape(gm,aFace1)->tag();
+        numFaceSlave  = gm->getOCCInternals()->getFaceForOCCShape(gm,aFace2)->tag();
         //Msg::Info("Face %d (slave) is most likely Face %d (master) translated by (%.2e,%.2e,%.2e)!",
         //           numFaceSlave,numFaceMaster,FaceTranslationVector[0],FaceTranslationVector[1],
         //           FaceTranslationVector[2]);
@@ -1206,7 +1188,7 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
         for (TopExp_Explorer aEdgeExplorer1(aFace1,TopAbs_EDGE); aEdgeExplorer1.More();
              aEdgeExplorer1.Next()) {
           TopoDS_Edge aEdge1 = TopoDS::Edge(aEdgeExplorer1.Current());
-          int numEdgeMaster  = gm->getOCCInternals()->getOCCEdgeByNativePtr(gm,aEdge1)->tag();
+          int numEdgeMaster  = gm->getOCCInternals()->getEdgeForOCCShape(gm,aEdge1)->tag();
           EdgeListMaster[i1] = numEdgeMaster;
           //i2=0;
           for (TopExp_Explorer aEdgeExplorer2(aFace2,TopAbs_EDGE); aEdgeExplorer2.More();
@@ -1224,7 +1206,7 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
                && IsEqualG(BarycenterEdge1.X()+FaceTranslationVector[0],BarycenterEdge2.X())
                && IsEqualG(BarycenterEdge1.Y()+FaceTranslationVector[1],BarycenterEdge2.Y())
                && IsEqualG(BarycenterEdge1.Z()+FaceTranslationVector[2],BarycenterEdge2.Z())){
-              int numEdgeSlave   = gm->getOCCInternals()->getOCCEdgeByNativePtr(gm,aEdge2)->tag();
+              int numEdgeSlave   = gm->getOCCInternals()->getEdgeForOCCShape(gm,aEdge2)->tag();
               EdgeListSlave[i1]  = numEdgeSlave;
             }
           }
@@ -1250,6 +1232,9 @@ void OCCFactory::setPeriodicPairOfFaces(GModel *gm, int numFaceMaster,
                                         int numFaceSlave,
                                         std::vector<int> EdgeListSlave)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
   if (EdgeListMaster.size() != EdgeListSlave.size()){
     Msg::Error("Slave/Master faces don't have the same number of edges!");
   }
@@ -1263,31 +1248,27 @@ void OCCFactory::setPeriodicPairOfFaces(GModel *gm, int numFaceMaster,
       edgeCounterparts[*siter] = *miter;
     }
 
-    Surface *s_slave = FindSurface(abs(numFaceSlave));
-    if(s_slave){
-      s_slave->master = numFaceMaster;
-      s_slave->edgeCounterparts = edgeCounterparts;
-    }
-    else{
-      GFace *gf = GModel::current()->getFaceByTag(abs(numFaceSlave));
-      if (gf) {
-        GFace *master = GModel::current()->getFaceByTag(abs(numFaceMaster));
-        gf->setMeshMaster(master,edgeCounterparts);
-      }
-      else Msg::Error("Slave surface %d not found", numFaceSlave);
+    GFace *gf = GModel::current()->getFaceByTag(abs(numFaceSlave));
+    if (gf) {
+      GFace *master = GModel::current()->getFaceByTag(abs(numFaceMaster));
+      gf->setMeshMaster(master,edgeCounterparts);
     }
+    else Msg::Error("Slave surface %d not found", numFaceSlave);
   }
 }
 
 void OCCFactory::fillet(GModel *gm, std::vector<int> edges, double radius)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
   try{
     std::vector<TopoDS_Edge> edgesToFillet;
     for (unsigned i = 0; i < edges.size(); i++){
       GEdge *ed = gm->getEdgeByTag(edges[i]);
       if (ed){
-	OCCEdge *occed = dynamic_cast<OCCEdge*>(ed);
-	if (occed)edgesToFillet.push_back(occed->getTopoDS_Edge());
+  OCCEdge *occed = dynamic_cast<OCCEdge*>(ed);
+  if (occed)edgesToFillet.push_back(occed->getTopoDS_Edge());
       }
     }
     gm->_occ_internals->fillet(edgesToFillet, radius);
@@ -1302,6 +1283,9 @@ void OCCFactory::fillet(GModel *gm, std::vector<int> edges, double radius)
 
 void OCCFactory::translate(GModel *gm, std::vector<double> dx, int addToTheModel)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
   gp_Trsf transformation;
   transformation.SetTranslation(gp_Pnt (0,0,0),gp_Pnt (dx[0],dx[1],dx[2]));
   BRepBuilderAPI_Transform aTransformation(gm->_occ_internals->getShape(),
@@ -1317,6 +1301,9 @@ void OCCFactory::translate(GModel *gm, std::vector<double> dx, int addToTheModel
 void OCCFactory::rotate(GModel *gm, std::vector<double> p1, std::vector<double> p2,
                         double angle, int addToTheModel)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
   const double x1 = p1[0];
   const double y1 = p1[1];
   const double z1 = p1[2];
@@ -1324,9 +1311,6 @@ void OCCFactory::rotate(GModel *gm, std::vector<double> p1, std::vector<double>
   const double y2 = p2[1];
   const double z2 = p2[2];
 
-  if (!gm->_occ_internals)
-    gm->_occ_internals = new OCC_Internals;
-
   gp_Trsf transformation;
 
   gp_Vec direction(gp_Pnt(x1, y1, z1), gp_Pnt(x2, y2, z2));
@@ -1342,9 +1326,39 @@ void OCCFactory::rotate(GModel *gm, std::vector<double> p1, std::vector<double>
   gm->_occ_internals->buildGModel(gm);
 }
 
+void OCCFactory::dilate(GModel *gm, std::vector<double> p, std::vector<double> s,
+                        int addToTheModel)
+{
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
+  const double x = p[0];
+  const double y = p[1];
+  const double z = p[2];
+
+  const double a = s[0];
+  const double b = s[1];
+  const double c = s[2];
+
+  gp_GTrsf transformation;
+  transformation.SetVectorialPart(gp_Mat(a, 0, 0, 0, b, 0, 0, 0, c));
+  transformation.SetTranslationPart(gp_XYZ(x,y,z));
+  BRepBuilderAPI_GTransform aTransformation(gm->_occ_internals->getShape(),
+                                           transformation, Standard_False);
+  TopoDS_Shape temp = aTransformation.Shape();
+  if (!addToTheModel) gm->_occ_internals->loadShape(&temp);
+  else gm->_occ_internals->buildShapeFromLists(temp);
+  gm->destroy();
+  gm->_occ_internals->buildLists();
+  gm->_occ_internals->buildGModel(gm);
+}
+
 std::vector<GFace *> OCCFactory::addRuledFaces(GModel *gm,
                                                std::vector< std::vector<GEdge *> > wires)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
   std::vector<GFace*> faces;
   Standard_Boolean anIsSolid = Standard_False;
   Standard_Boolean anIsRuled = Standard_True;
@@ -1356,7 +1370,7 @@ std::vector<GFace *> OCCFactory::addRuledFaces(GModel *gm,
       GEdge *ge = wires[i][j];
       OCCEdge *occe = dynamic_cast<OCCEdge*>(ge);
       if (occe){
-	wire_maker.Add(occe->getTopoDS_Edge());
+  wire_maker.Add(occe->getTopoDS_Edge());
       }
     }
     aGenerator.AddWire (wire_maker.Wire());
@@ -1380,6 +1394,9 @@ std::vector<GFace *> OCCFactory::addRuledFaces(GModel *gm,
 GFace *OCCFactory::addFace(GModel *gm, std::vector<GEdge *> edges,
                            std::vector< std::vector<double > > points)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
   BRepOffsetAPI_MakeFilling aGenerator;
 
   for (unsigned i = 0; i < edges.size(); i++) {
@@ -1402,6 +1419,8 @@ GFace *OCCFactory::addFace(GModel *gm, std::vector<GEdge *> edges,
 
 GFace *OCCFactory::addPlanarFace(GModel *gm, std::vector< std::vector<GEdge *> > wires)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
 
   std::set<GVertex*> verts;
   for (unsigned i = 0; i < wires.size(); i++) {
@@ -1429,7 +1448,7 @@ GFace *OCCFactory::addPlanarFace(GModel *gm, std::vector< std::vector<GEdge *> >
       GEdge *ge = wires[i][j];
       OCCEdge *occe = dynamic_cast<OCCEdge*>(ge);
       if (occe){
-	wire_maker.Add(occe->getTopoDS_Edge());
+  wire_maker.Add(occe->getTopoDS_Edge());
       }
     }
     TopoDS_Wire myWire = wire_maker.Wire();
@@ -1451,6 +1470,9 @@ GFace *OCCFactory::addPlanarFace(GModel *gm, const std::vector<std::vector<GEdge
 
 GEntity *OCCFactory::addPipe(GModel *gm, GEntity *base, std::vector<GEdge *> wire)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
   BRepBuilderAPI_MakeWire wire_maker;
   for (unsigned j = 0; j < wire.size(); j++) {
     GEdge *ge = wire[j];
@@ -1483,69 +1505,42 @@ GEntity *OCCFactory::addPipe(GModel *gm, GEntity *base, std::vector<GEdge *> wir
   return ret;
 }
 
+GEntity *OCCFactory::addThruSections(GModel *gm, std::vector<std::vector<GEdge *> > wire)
+{
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
+  BRepOffsetAPI_ThruSections aGenerator(Standard_True); // create solid
+  for (unsigned i = 0; i < wire.size(); i++) {
+    BRepBuilderAPI_MakeWire wire_maker;
+    for (unsigned j = 0; j < wire[i].size(); j++) {
+      GEdge *ge = wire[i][j];
+      OCCEdge *occe = dynamic_cast<OCCEdge*>(ge);
+      if (occe){
+        wire_maker.Add(occe->getTopoDS_Edge());
+      }
+    }
+    aGenerator.AddWire(wire_maker.Wire());
+  }
+  aGenerator.CheckCompatibility(Standard_False);
+  aGenerator.Build();
+  TopoDS_Solid result = TopoDS::Solid(aGenerator.Shape());
+  GEntity *ret = gm->_occ_internals->addRegionToModel(gm, result);
+  return ret;
+}
+
 void OCCFactory::healGeometry(GModel *gm, double tolerance)
 {
+  if (!gm->_occ_internals)
+    gm->_occ_internals = new OCC_Internals;
+
   if (tolerance < 0.)
     tolerance = CTX::instance()->geom.tolerance;
   if (!gm || !gm->_occ_internals)
     return;
-  //gm->_occ_internals->healGeometry(tolerance, false, false, false, true, false, false);
-  gm->_occ_internals->healGeometry(tolerance, true, true, true, true, true, true);
+  gm->_occ_internals->_healGeometry(tolerance, true, true, true, true, true, true);
   gm->_occ_internals->buildLists();
   gm->_occ_internals->buildGModel(gm);
 }
 
-//Prepare SGEOM integration
-#if defined(HAVE_SGEOM) && defined(HAVE_OCC)
-
-#include "SGEOMIncludes.h"
-
-GVertex* SGEOMFactory::addVertex(GModel *gm,double x, double y, double z, double lc)
-{
-  //if (!gm->_sgeom_internals)
-  //  gm->_sgeom_internals = new SGEOM_Internals;
-
-  //gp_Pnt aPnt;
-  //aPnt = gp_Pnt(x, y, z);
-  //BRepBuilderAPI_MakeVertex mkVertex(aPnt);
-  //TopoDS_Vertex occv = mkVertex.Vertex();
-
-  //return gm->_occ_internals->addVertexToModel(gm, occv);
-
-  Msg::Error("addVertex not implemented yet for SGEOMFactory");
-  return 0;
-}
-
-GEdge* SGEOMFactory::addLine(GModel *gm,GVertex *v1, GVertex *v2)
-{
-  Msg::Error("addLine not implemented yet for SGEOMFactory");
-  return 0;
-}
-
-GFace* SGEOMFactory::addPlanarFace(GModel *gm, std::vector<std::vector<GEdge *> > edges)
-{
-  Msg::Error("addPlanarFace not implemented yet for SGEOMFactory");
-  return 0;
-}
-
-GFace* SGEOMFactory::addPlanarFace(GModel *gm, const std::vector<std::vector<GEdgeSigned> > &edges)
-{
-  Msg::Error("addPlanarFace with oriented edges not implemented yet for SGEOMFactory");
-  return 0;
-}
-
-GRegion* SGEOMFactory::addVolume(GModel *gm, std::vector<std::vector<GFace *> > faces)
-{
-  Msg::Error("addVolume not implemented yet for SGEOMFactory");
-  return 0;
-}
-
-void SGEOMFactory::healGeometry(GModel *gm, double tolerance)
-{
-  Msg::Error("healGeometry not implemented yet for SGEOMFactory");
-  return 0;
-}
-
-#endif
-
 #endif
