@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -29,9 +29,9 @@
 
 #define FAST_ELEMENTS 1
 
-extern void writeMSHPeriodicNodes (FILE *fp, std::vector<GEntity*> &entities, bool renumber);
-
+extern void writeMSHPeriodicNodes(FILE *fp, std::vector<GEntity*> &entities, bool renumber);
 extern void readMSHPeriodicNodes(FILE *fp, GModel *gm);
+extern void writeMSHEntities(FILE *fp, GModel *gm);
 
 static bool getVertices(int num, int *indices, std::map<int, MVertex*> &map,
                         std::vector<MVertex*> &vertices)
@@ -719,7 +719,7 @@ int GModel::_readMSH2(const std::string &name)
 
   fclose(fp);
 
-  alignPeriodicBoundaries();
+  if(!CTX::instance()->mesh.ignorePeriodicity) alignPeriodicBoundaries();
 
   return postpro ? 2 : 1;
 }
@@ -927,6 +927,9 @@ int GModel::_writeMSH2(const std::string &name, double version, bool binary,
       fprintf(fp, "$EndPhysicalNames\n");
     }
 
+    if (CTX::instance()->mesh.saveTopology)
+      writeMSHEntities(fp, this);
+
     if (saveParametric)
       fprintf(fp, "$ParametricNodes\n");
     else
