@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -106,9 +106,9 @@ bool isElementVisible(MElement *ele)
     if(CTX::instance()->mesh.qualityType == 3)
       q = ele->distoShapeMeasure();
     else if(CTX::instance()->mesh.qualityType == 2)
-      q = ele->rhoShapeMeasure();
-    else if(CTX::instance()->mesh.qualityType == 1)
       q = ele->gammaShapeMeasure();
+    else if(CTX::instance()->mesh.qualityType == 1)
+      q = ele->minSIGEShapeMeasure();
     else
       q = ele->minSICNShapeMeasure();
     if(q < CTX::instance()->mesh.qualityInf ||
