@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -221,6 +221,18 @@ void GRegion::setVisibility(char val, bool recursive)
   }
 }
 
+void GRegion::setColor(unsigned int val, bool recursive)
+{
+  GEntity::setColor(val);
+  if(recursive){
+    std::list<GFace*>::iterator it = l_faces.begin();
+    while(it != l_faces.end()){
+      (*it)->setColor(val, recursive);
+      ++it;
+    }
+  }
+}
+
 std::string GRegion::getAdditionalInfoString()
 {
   std::ostringstream sstream;
@@ -257,6 +269,18 @@ void GRegion::writeGEO(FILE *fp)
     fprintf(fp, "Volume(%d) = {%d};\n", tag(), tag());
   }
 
+  for(std::list<GFace*>::iterator it = embedded_faces.begin();
+      it != embedded_faces.end(); it++)
+    fprintf(fp, "Surface {%d} In Volume {%d};\n", (*it)->tag(), tag());
+
+  for(std::list<GEdge*>::iterator it = embedded_edges.begin();
+      it != embedded_edges.end(); it++)
+    fprintf(fp, "Line {%d} In Volume {%d};\n", (*it)->tag(), tag());
+
+  for(std::list<GVertex*>::iterator it = embedded_vertices.begin();
+      it != embedded_vertices.end(); it++)
+    fprintf(fp, "Point {%d} In Volume {%d};\n", (*it)->tag(), tag());
+
   if(meshAttributes.method == MESH_TRANSFINITE){
     fprintf(fp, "Transfinite Volume {%d}", tag());
     if(meshAttributes.corners.size()){
