@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -473,8 +473,11 @@ class MHexahedron27 : public MHexahedron {
  *
  */
 
+typedef std::vector<int> indicesReversed;
 
 class MHexahedronN : public MHexahedron {
+  static std::map<int, indicesReversed> _order2indicesReversedHex;
+
  protected:
   const char _order;
   std::vector<MVertex*> _vs;
@@ -579,10 +582,7 @@ class MHexahedronN : public MHexahedron {
   }
   virtual int getNumFacesRep(bool curved);
   virtual void getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n);
-  virtual void reverse()
-  {
-    Msg::Error("Reverse not implemented yet for MHexahedronN");
-  }
+  virtual void reverse();
   virtual void getNode(int num, double &u, double &v, double &w) const
   {
     num < 8 ? MHexahedron::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
