@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -309,8 +309,11 @@ class MTetrahedron10 : public MTetrahedron {
  *
  */
 
+typedef std::vector<int> indicesReversed;
+
 class MTetrahedronN : public MTetrahedron {
-  static const std::vector<int> &_getReverseIndices(int order);
+  static std::map<int, indicesReversed> _order2indicesReversedTet;
+
  protected:
   std::vector<MVertex *> _vs;
   const char _order;
@@ -412,16 +415,7 @@ class MTetrahedronN : public MTetrahedron {
     Msg::Error("no tag matches a p%d tetrahedron with %d vertices", _order, 4+_vs.size());
     return 0;
   }
-  virtual void reverse()
-  {
-    MVertex *tmp;
-    tmp = _v[1]; _v[1] = _v[2]; _v[2] = tmp;
-    std::vector<MVertex*> inv(_vs.size());
-    std::vector<int> reverseIndices = _getReverseIndices(_order);
-    for (unsigned int i = 0; i< _vs.size(); i++)
-      inv[i] = _vs[reverseIndices[i + 4] - 4];
-    _vs = inv;
-  }
+  virtual void reverse();
   virtual void getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n);
   virtual int getNumEdgesRep(bool curved);
   virtual void getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n);
