@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -340,14 +340,7 @@ class MTriangleN : public MTriangle {
   {
     return (_order == 2) ? 22 : MTriangle::getTypeForVTK();
   }
-  virtual void reverse()
-  {
-    MVertex *tmp;
-    tmp = _v[1]; _v[1] = _v[2]; _v[2] = tmp;
-    std::vector<MVertex*> inv;
-    inv.insert(inv.begin(), _vs.rbegin(), _vs.rend());
-    _vs = inv;
-  }
+  virtual void reverse();
   virtual void getNode(int num, double &u, double &v, double &w) const
   {
     num < 3 ? MTriangle::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
@@ -359,9 +352,9 @@ class MTriangleN : public MTriangle {
 };
 
 template <class T>
-void sort3(T *t[3])
+void sort3(T t[3])
 {
-  T *temp;
+  T temp;
   if(t[0] > t[1]){
     temp = t[1];
     t[1] = t[0];
@@ -383,8 +376,8 @@ struct compareMTriangleLexicographic
 {
   bool operator () (MTriangle *t1, MTriangle *t2) const
   {
-    MVertex *_v1[3] = {t1->getVertex(0), t1->getVertex(1), t1->getVertex(2)};
-    MVertex *_v2[3] = {t2->getVertex(0), t2->getVertex(1), t2->getVertex(2)};
+    int _v1[3] = {t1->getVertex(0)->getNum(), t1->getVertex(1)->getNum(), t1->getVertex(2)->getNum()};
+    int _v2[3] = {t2->getVertex(0)->getNum(), t2->getVertex(1)->getNum(), t2->getVertex(2)->getNum()};
     sort3(_v1);
     sort3(_v2);
     if(_v1[0] < _v2[0]) return true;
