@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -9,13 +9,17 @@
 #include "GmshConfig.h"
 #include "GEdge.h"
 #include "GModel.h"
-#include "OCCVertex.h"
 #include "Range.h"
 
 class OCCFace;
 
 #if defined(HAVE_OCC)
 
+#include <TopoDS_Edge.hxx>
+#include <Geom_Curve.hxx>
+#include <Geom2d_Curve.hxx>
+#include <BRep_Tool.hxx>
+
 class OCCEdge : public GEdge {
  protected:
   TopoDS_Edge _replacement;
