@@ -70,6 +70,12 @@ class SVector3 {
     P[0] = v; P[1] = v; P[2] = v;
     return *this;
   }
+  SVector3 & operator = (const SVector3& a)
+  {
+    P[0] = a[0]; P[1] = a[1]; P[2] = a[2];
+    return *this;
+  }
+  
   operator double *() { return P; }
   void print(std::string name="") const
   {
