@@ -3,7 +3,7 @@
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
-#include <queue> 
+#include <queue>
 #include <stdlib.h>
 #include "GmshConfig.h"
 
@@ -15,12 +15,11 @@
 #include "discreteEdge.h"
 #include "MTriangle.h"
 #include "MEdge.h"
+#include "GModelIO_GEO.h"
 #include "Geo.h"
 #include "Context.h"
 #include "OS.h"
 #include "ANN/ANN.h"
-#include "laplaceTerm.h"
-#include "convexLaplaceTerm.h"
 #include "convexCombinationTerm.h"
 
 #if defined(HAVE_MESH)
@@ -291,8 +290,8 @@ discreteDiskFace::discreteDiskFace(GFace *gf, triangulation* diskTriangulation,
   orderVertices(_totLength, _U0, _coords);
   parametrize();
   buildOct(CAD);
-  printParamMesh();
-  Msg::Info("Parameterization done (GFace %d)",gf->tag());
+  //putOnView(gf->tag(),diskTriangulation->idNum,true,true);
+  //printParamMesh();
 }
 /*end BUILDER*/
 
@@ -301,7 +300,7 @@ discreteDiskFace::~discreteDiskFace()
   triangles.clear();
   if (_ddft) delete[] _ddft;
   if (oct) Octree_Delete(oct);
-  if(geoTriangulation) delete geoTriangulation;
+  if (geoTriangulation) delete geoTriangulation;
 }
 
 void discreteDiskFace::buildOct(std::vector<GFace*> *CAD) const
@@ -313,11 +312,11 @@ void discreteDiskFace::buildOct(std::vector<GFace*> *CAD) const
   const int maxElePerBucket = 15;
   oct = Octree_Create(maxElePerBucket, origin, ssize, discreteDiskFaceBB,
 		      discreteDiskFaceCentroid, discreteDiskFaceInEle);
-  
-  _ddft = new discreteDiskFaceTriangle[discrete_triangles.size()/*-geoTriangulation->fillingHoles.size()*/];
+
+  _ddft = new discreteDiskFaceTriangle[discrete_triangles.size()];
   int c = 0;
   for(unsigned int i = 0; i < discrete_triangles.size(); ++i){
-    //    if(geoTriangulation->fillingHoles.find(i)==geoTriangulation->fillingHoles.end()){
+
     MElement *t = discrete_triangles[i];
     discreteDiskFaceTriangle* my_ddft = &_ddft[c++];
     my_ddft->p.resize(_n);
@@ -328,53 +327,17 @@ void discreteDiskFace::buildOct(std::vector<GFace*> *CAD) const
     my_ddft->tri = t;
 
     Octree_Insert(my_ddft, oct);
-    //    }
+
   }
   Octree_Arrange(oct);
 }
 
-/*
-void discreteDiskFace::OneToOneParametrization () {
-  v2t_cont adj;
-  buildVertexToTriangle(discrete_triangles, adj);
-  int count=0;
-  {
-    v2t_cont :: iterator it = adj.begin();
-    while (it != adj.end()) {
-      it->first->setIndex(count++);
-      ++it;
-    }
-  }  
-  double *rhs = new double [2*count];
-  double *x   = new double [2*count];
-  {
-    for(size_t i = 0; i < _U0.size(); i++){
-      MVertex *v = _U0[i];
-      const double theta = 2 * M_PI * _coords[i];
-      myAssemblerU.fixVertex(v, 0, 1,cos(theta));
-      myAssemblerV.fixVertex(v, 0, 1,sin(theta));
-    }  
-  }
 
-  {
-    v2t_cont :: iterator it = adj.begin();
-    while (it != adj.end()) {
-      it->first->setIndex(i++);
-      ++it;
-    }
-  }  
-
-  std::vector<int> ij;
-  std::vector<double> val;
-  
-}
-*/
-
-bool discreteDiskFace::parametrize()// const
+bool discreteDiskFace::parametrize() const
 { // #improveme
 
   linearSystem<double> * lsys_u, *lsys_v;
-  
+
 
 #ifdef HAVE_MUMPS
   lsys_u = new linearSystemMUMPS<double>;
@@ -389,18 +352,12 @@ bool discreteDiskFace::parametrize()// const
 #endif
   dofManager<double> myAssemblerU(lsys_u);   // hashing
   dofManager<double> myAssemblerV(lsys_v);
-  
+
   for(size_t i = 0; i < _U0.size(); i++){
     MVertex *v = _U0[i];
     const double theta = 2 * M_PI * _coords[i];
-    if(i%_order==0){
-      myAssemblerU.fixVertex(v, 0, 1,cos(theta));
-      myAssemblerV.fixVertex(v, 0, 1,sin(theta));
-    }
-    else{//#TEST
-      myAssemblerU.fixVertex(v, 0, 1,1./_order*((_order-(i%_order))*cos(2*M_PI*_coords[i-(i%_order)])+(i%_order)*cos(2*M_PI*_coords[i+(_order-(i%_order))])));
-      myAssemblerV.fixVertex(v, 0, 1,1./_order*((_order-(i%_order))*sin(2*M_PI*_coords[i-(i%_order)])+(i%_order)*sin(2*M_PI*_coords[i+(_order-(i%_order))])));
-    }
+    myAssemblerU.fixVertex(v, 0, 1,cos(theta));
+    myAssemblerV.fixVertex(v, 0, 1,sin(theta));
   }
 
   for(size_t i = 0; i < discrete_triangles.size(); ++i){
@@ -458,9 +415,7 @@ bool discreteDiskFace::parametrize()// const
   return true;
 }
 
-//void discreteDiskFace::checklsys(linearSystemCSR<double>* lsys,dofManager<double>* myAssembler,int U)
-//{
-//}
+
 
 void discreteDiskFace::getTriangleUV(const double u,const double v,
 				     discreteDiskFaceTriangle **mt,
@@ -544,6 +499,7 @@ bool discreteDiskFace::checkOrientationUV()
   }
 }
 
+// for second order parameterization
 void discreteDiskFace::optimize()
 { // #improve . . .
 #if defined(HAVE_OPTHOM)
@@ -613,18 +569,18 @@ void discreteDiskFace::optimize()
   lines.push_back(new MLine(sp2mv[coordinates[_U0[_U0.size()-1]]],sp2mv[coordinates[_U0[0]]]));
   de->setTopo(lines);
   de->createGeometry();// !!!! setTopo ... MLine's
- 
-  
-  
+
+
+
   // optimization
   if(_order >1)
     HighOrderMeshOptimizer(paramDisk, optParams);
   else
     MeshQualityOptimizer(paramDisk,opt);
 
- 
-  
-  
+
+
+
   // update the parametrization
   paramTriangles = e2e[0];
   for(unsigned int i=0; i< paramTriangles.size(); i++){
@@ -647,10 +603,10 @@ void discreteDiskFace::optimize()
   }
   dgf->mesh_vertices.clear();
   dgf->mesh_vertices = newMV;
-  
+
   // cleaning
   delete paramDisk;
-  
+
 #endif
 }
 
@@ -952,6 +908,8 @@ GPoint discreteDiskFace::intersectionWithCircle(const SVector3 &n1, const SVecto
 						const SVector3 &p, const double &R,
 						double uv[2]) const
 {
+
+
   SVector3 n = crossprod(n1,n2);
   n.normalize();
   //  printf("n %g %g %g\n",n.x(), n.y(), n.z());
@@ -982,7 +940,7 @@ GPoint discreteDiskFace::intersectionWithCircle(const SVector3 &n1, const SVecto
     m[0][0] = n.y();
     m[0][1] = n.z();
     m[1][0] = t.y();
-    m[1][1] = t.z();      
+    m[1][1] = t.z();
     if (fabs(det2x2(m)) > 1.e-12){
       sys2x2(m,rhs,r);
       x0 = SVector3(0,r[0],r[1]);
@@ -991,17 +949,17 @@ GPoint discreteDiskFace::intersectionWithCircle(const SVector3 &n1, const SVecto
       m[0][0] = n.x();
       m[0][1] = n.z();
       m[1][0] = t.x();
-      m[1][1] = t.z();      
+      m[1][1] = t.z();
       if (fabs(det2x2(m)) > 1.e-12){
-	sys2x2(m,rhs,r);	
+	sys2x2(m,rhs,r);
 	x0 = SVector3(r[0],0,r[1]);
       }
       else {
 	m[0][0] = n.x();
 	m[0][1] = n.y();
 	m[1][0] = t.x();
-	m[1][1] = t.y();      
-	if (sys2x2(m,rhs,r))	{	  
+	m[1][1] = t.y();
+	if (sys2x2(m,rhs,r))	{
 	  x0 = SVector3(r[0],r[1],0);
 	}
 	else{
@@ -1009,8 +967,8 @@ GPoint discreteDiskFace::intersectionWithCircle(const SVector3 &n1, const SVecto
 	  continue;
 	}
       }
-    }    
-    
+    }
+
     const double a = 1.0;
     const double b = -2*dot(d,p-x0);
     const double c = dot(p-x0,p-x0) - R*R;
@@ -1047,7 +1005,7 @@ GPoint discreteDiskFace::intersectionWithCircle(const SVector3 &n1, const SVecto
   //  printf("Point(1) = {%g,%g,%g};\n",p.x(),p.y(),p.z());
   //  printf("Point(2) = {%g,%g,%g};\n",p.x()+d*n1.x(),p.y()+d*n1.y(),p.z()+d*n1.z());
   //  printf("Point(3) = {%g,%g,%g};\n",p.x()+d*n2.x(),p.y()+d*n2.y(),p.z()+d*n2.z());
-  
+
   //  //  printf("Circle(4) = {2,1,3};\n");
   //  printf("{%g,%g,%g};\n",n1.x(),n1.y(),n1.z());
   //  printf("{%g,%g,%g};\n",n2.x(),n2.y(),n2.z());
@@ -1064,6 +1022,7 @@ GPoint discreteDiskFace::intersectionWithCircle2(const SVector3 &n1, const SVect
 						const SVector3 &p, const double &d,
 						double uv[2]) const
 {
+
   // n2 is exterior
   SVector3 n = crossprod(n1,n2);
   n.normalize();
@@ -1126,7 +1085,7 @@ GPoint discreteDiskFace::intersectionWithCircle2(const SVector3 &n1, const SVect
       const double ta = (-b + sign*sqrt(delta)) / (2.*a);
       const double tb = (-b - sign*sqrt(delta)) / (2.*a);
       SVector3 s[2] =  {q + m * ta, q + m * tb};
-      for (int IT=0;IT<2;IT++){	
+      for (int IT=0;IT<2;IT++){
 	double mat[2][2], b[2],uv[2];
 	mat[0][0] = dot(t1,t1);
 	mat[1][1] = dot(t2,t2);
@@ -1153,7 +1112,7 @@ GPoint discreteDiskFace::intersectionWithCircle2(const SVector3 &n1, const SVect
   //  printf("Point(1) = {%g,%g,%g};\n",p.x(),p.y(),p.z());
   //  printf("Point(2) = {%g,%g,%g};\n",p.x()+d*n1.x(),p.y()+d*n1.y(),p.z()+d*n1.z());
   //  printf("Point(3) = {%g,%g,%g};\n",p.x()+d*n2.x(),p.y()+d*n2.y(),p.z()+d*n2.z());
-  
+
   //  //  printf("Circle(4) = {2,1,3};\n");
   //  printf("{%g,%g,%g};\n",n1.x(),n1.y(),n1.z());
   //  printf("{%g,%g,%g};\n",n2.x(),n2.y(),n2.z());
@@ -1205,7 +1164,7 @@ static double computeDistanceLinePoint (MVertex *v1, MVertex *v2, MVertex *v){
 }
 */
 inline double computeDistance (MVertex *v1, double d1, MVertex *v2, double d2, MVertex *v){
-  
+
   //       o------------a
   //
   //
@@ -1331,11 +1290,11 @@ double triangulation::geodesicDistance ()
     if (Fixed.size() == v2t.size())break;
   }
 
-  /*
+
   char name[256];
-  sprintf(name,"geodesicDistance%d.pos",idNum);
+  sprintf(name,"geodesicDistance%d.pos",iter);
   FILE *f = fopen(name,"w");
-  fprintf(f,"View \"\"{\n");
+  fprintf(f,"View \"%d\"{\n",iter);
   for (unsigned int i=0;i<tri.size();i++){
     double d0 = Fixed[tri[i]->getVertex(0)];
     double d1 = Fixed[tri[i]->getVertex(1)];
@@ -1348,7 +1307,7 @@ double triangulation::geodesicDistance ()
   }
   fprintf(f,"};\n");
   fclose(f);
-  */
+
 
   return CLOSEST;
 }
@@ -1360,31 +1319,41 @@ void discreteDiskFace::printAtlasMesh()
   char buffer[256];
   sprintf(buffer,"atlas_mesh%d.msh",initialTriangulation->idNum);
   FILE* pmesh = Fopen(buffer,"w");
-	
+
   std::set<MVertex*> meshvertices;
-	
+
   for(unsigned int i=0; i<initialTriangulation->tri.size(); ++i){
     MElement* tri = initialTriangulation->tri[i];
     for(unsigned int j=0; j<3; j++)
-      if (meshvertices.find(tri->getVertex(j))==meshvertices.end()) meshvertices.insert(tri->getVertex(j));
+      if (meshvertices.find(tri->getVertex(j))==meshvertices.end())
+        meshvertices.insert(tri->getVertex(j));
   }
-	
-  fprintf(pmesh,"$MeshFormat\n2.2 0 8\n$EndMeshFormat\n$Nodes\n%u\n",(unsigned int)meshvertices.size());
+
+  fprintf(pmesh,"$MeshFormat\n2.2 0 8\n$EndMeshFormat\n$Nodes\n%d\n",
+          (int)meshvertices.size());
   int count = 1;
-  for(std::set<MVertex*>::iterator it = meshvertices.begin(); it!=meshvertices.end(); ++it){
+  for(std::set<MVertex*>::iterator it = meshvertices.begin();
+      it!=meshvertices.end(); ++it){
     fprintf(pmesh,"%d %f %f %f\n",count,(*it)->x(),(*it)->y(),(*it)->z());
     mv2int[*it] = count;
     count++;
   }
-  fprintf(pmesh,"$EndNodes\n$Elements\n%u\n",(unsigned int)initialTriangulation->tri.size());
+  fprintf(pmesh,"$EndNodes\n$Elements\n%d\n",
+          (int)(initialTriangulation->tri.size() -
+                initialTriangulation->fillingHoles.size()));
+  unsigned int mycount = 0;//#####
   for(unsigned int i=0; i<initialTriangulation->tri.size(); i++){
+    //std::set<int>::iterator it = initialTriangulation->fillingHoles.find(i);
+    //if (it == initialTriangulation->fillingHoles.end()){
     MElement* tri = initialTriangulation->tri[i];
-    fprintf(pmesh,"%d 2 2 0 0",i+1);
+    fprintf(pmesh,"%d 2 2 0 %d",mycount+1,initialTriangulation->idNum);//#####
     for(int j=0; j<3; j++){
       MVertex* mv = tri->getVertex(j);
       fprintf(pmesh," %d",mv2int[mv]);
     }
     fprintf(pmesh,"\n");
+    mycount++;//###
+    //}
   }
   fprintf(pmesh,"$EndElements\n");
   fclose(pmesh);
