@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -17,6 +17,7 @@
 #include "MTetrahedron.h"
 #include "MHexahedron.h"
 #include "MPyramid.h"
+#include "GModelIO_GEO.h"
 #include "Geo.h"
 #include "OS.h"
 #include "Curvature.h"
@@ -31,7 +32,7 @@ discreteEdge::discreteEdge(GModel *model, int num, GVertex *_v0, GVertex *_v1)
   : GEdge(model, num, _v0, _v1)
 {
   createdTopo = false;
-  Curve *c = Create_Curve(num, MSH_SEGM_DISCRETE, 0, 0, 0, -1, -1, 0., 1.);
+  Curve *c = CreateCurve(num, MSH_SEGM_DISCRETE, 0, 0, 0, -1, -1, 0., 1.);
   Tree_Add(model->getGEOInternals()->Curves, &c);
   CreateReversedCurve(c);
 }
@@ -70,6 +71,7 @@ void discreteEdge::orderMLines()
        it != segments.end(); ++it){
     MVertex *vL = (*it)->getVertex(0);
     MVertex *vR = (*it)->getVertex(1);
+
     std::map<MVertex*,MLine*>::iterator it1 = boundv.find(vL);
     if (it1 == boundv.end()) boundv.insert(std::make_pair(vL,*it));
     else  boundv.erase(it1);
@@ -78,6 +80,7 @@ void discreteEdge::orderMLines()
     else boundv.erase(it2);
   }
   // find the first MLine and erase it from the list segments
+
   MLine *firstLine;
   if (boundv.size() == 2){   // non periodic
     firstLine = (boundv.begin())->second;
@@ -141,7 +144,7 @@ void discreteEdge::orderMLines()
 
   //lines is now a list of ordered MLines
   lines = _m;
-  
+
   //mesh_vertices
   mesh_vertices.clear();
   for (unsigned int i = 0; i < lines.size()-1; ++i){
@@ -153,7 +156,7 @@ void discreteEdge::orderMLines()
         mesh_vertices.end()) mesh_vertices.push_back(v2);
   }
 
-  
+
   //special case reverse orientation
   if (lines.size() < 2) return;
   if (_orientation[0] && lines[0]->getVertex(1) != lines[1]->getVertex(1)
@@ -214,7 +217,6 @@ void discreteEdge::setBoundVertices()
     v1 = bound_vertices[1];
   }
   else if (boundv.size() == 0){
-    //printf("closed loop for discrete Edge =%d \n", this->tag());
     GVertex* bound_vertex;
     std::vector<MLine*>::const_iterator it = lines.begin();
     MVertex* vE = (*it)->getVertex(0);
@@ -366,7 +368,7 @@ void discreteEdge::parametrize(std::map<MVertex*, MVertex*>& old2new)
       _pars.push_back(i);
     }
   }
-  
+
   std::vector<MVertex* > newVertices;
   std::vector<MLine*> newLines;
 
@@ -394,7 +396,7 @@ void discreteEdge::parametrize(std::map<MVertex*, MVertex*>& old2new)
 
   mesh_vertices = newVertices;
   lines.clear();
-  lines = newLines;  
+  lines = newLines;
 }
 
 
@@ -453,7 +455,7 @@ GPoint discreteEdge::point(double par) const
   int iEdge;
   if(!getLocalParameter(par, iEdge, tLoc)) return GPoint();
 
-  
+
   double x, y, z;
   MVertex *vB = discrete_lines[iEdge]->getVertex(0);
   MVertex *vE = discrete_lines[iEdge]->getVertex(1);
@@ -467,7 +469,7 @@ GPoint discreteEdge::point(double par) const
 
 SVector3 discreteEdge::firstDer(double par) const
 {
-  
+
   double tLoc;
   int iEdge;
   if(!getLocalParameter(par, iEdge, tLoc)) return SVector3();
