@@ -1,41 +1,119 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
 
+#include <stack>
+#include <queue>
+#include <complex>
+#include <stdlib.h>
 #include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "discreteFace.h"
 #include "discreteDiskFace.h"
+#include "GModelIO_GEO.h"
 #include "Geo.h"
 #include "GFaceCompound.h"
 #include "Context.h"
 #include "OS.h"
-#include <stack>
-#include <queue>
-
+#include "meshPartitionObjects.h"
+#include "meshPartitionOptions.h"
+#include "meshPartition.h"
 #include "MPoint.h"
 
+#if defined(HAVE_PETSC)
+#include "linearSystemPETSc.h"
+#endif
+
 #if defined(HAVE_METIS)
 extern "C" {
 #include <metis.h>
 }
 #endif
 
+static inline double getAlpha(MTriangle* tri, int edj)
+{
+  double alpha;
+  if (edj==0)
+    alpha = 0.;
+  else{
+    MVertex *v0, *v1, *v2;
+
+    v0 = tri->getVertex(0);
+    v1 = tri->getVertex(1);
+    v2 = tri->getVertex(2);
+
+    SVector3 a(v1->x()-v0->x(),v1->y()-v0->y(),v1->z()-v0->z());
+    SVector3 b(v2->x()-v0->x(),v2->y()-v0->y(),v2->z()-v0->z());
+    SVector3 n = crossprod(a,b); n.normalize();
+
+    v0 = tri->getEdge(0).getSortedVertex(0);
+    v1 = tri->getEdge(0).getSortedVertex(1);
+    SVector3 U(v1->x()-v0->x(),v1->y()-v0->y(),v1->z()-v0->z());
+    U.normalize();
+    SVector3 V = crossprod(n,U); V.normalize();
+
+    v0 = tri->getEdge(edj).getSortedVertex(0);
+    v1 = tri->getEdge(edj).getSortedVertex(1);
+    SVector3 e(v1->x()-v0->x(),v1->y()-v0->y(),v1->z()-v0->z());
+    e.normalize();
+
+    alpha = std::atan2(dot(e,V),dot(e,U));
+  }
+
+  return alpha;
+
+}
+
+static inline void crouzeixRaviart(const std::vector<double> &U,std::vector<double> &F)
+{
+  F.resize(3);
+
+  double xsi[3] = {0.,1.,0.};
+  double eta[3] = {0.,0.,1.};
+  for(int i=0; i<3; i++)
+    F[i] =  U[0] * (1.-2.*eta[i]) + U[1] * (2.*(xsi[i]+eta[i])-1.) + U[2] * (1-2.*xsi[i]);
+}
+
 discreteFace::discreteFace(GModel *model, int num) : GFace(model, num)
 {
-  Surface *s = Create_Surface(num, MSH_SURF_DISCRETE);
+  Surface *s = CreateSurface(num, MSH_SURF_DISCRETE);
   Tree_Add(model->getGEOInternals()->Surfaces, &s);
   meshStatistics.status = GFace::DONE;
 }
 
-void discreteFace::setBoundEdges(GModel *gm, std::vector<int> tagEdges)
+void discreteFace::setBoundEdges(const std::vector<int> &tagEdges)
 {
-  for (std::vector<int>::iterator it = tagEdges.begin(); it != tagEdges.end(); it++){
-    GEdge *ge = gm->getEdgeByTag(*it);
-    l_edges.push_back(ge);
-    l_dirs.push_back(1);
-    ge->addFace(this);
+  for (unsigned int i = 0; i != tagEdges.size(); i++){
+    GEdge *ge = model()->getEdgeByTag(tagEdges[i]);
+    if(ge){
+      l_edges.push_back(ge);
+      l_dirs.push_back(1);
+      ge->addFace(this);
+    }
+    else{
+      Msg::Error("Unknown model edge %d", tagEdges[i]);
+    }
+  }
+}
+
+void discreteFace::setBoundEdges(const std::vector<int> &tagEdges,
+                                 const std::vector<int> &signEdges)
+{
+  if(signEdges.size() != tagEdges.size()){
+    Msg::Error("Wrong number of edge signs in setBoundEdges");
+    setBoundEdges(tagEdges);
+  }
+  for (unsigned int i = 0; i != tagEdges.size(); i++){
+    GEdge *ge = model()->getEdgeByTag(tagEdges[i]);
+    if(ge){
+      l_edges.push_back(ge);
+      l_dirs.push_back(signEdges[i]);
+      ge->addFace(this);
+    }
+    else{
+      Msg::Error("Unknown model edge %d", tagEdges[i]);
+    }
   }
 }
 
@@ -111,25 +189,35 @@ void discreteFace::secondDer(const SPoint2 &param,
 
 void discreteFace::createGeometry()
 {
+
   checkAndFixOrientation();
 
+
 #if defined(HAVE_SOLVER) && defined(HAVE_ANN)
+
+
+
   int order = 1;
   int nPart = 2;
-  double eta = 5/(2.*3.14);
+  double eta = .5*5/(2.*3.14);
   if (!_atlas.empty())return;
 
   double dtSplit = 0.0;
 
-  int id=1;
+  int id=1, iter=1;
   std::stack<triangulation*>  toSplit;
   std::vector<triangulation*> toParam;
 
   std::vector<MElement*> tem(triangles.begin(),triangles.end());
   triangulation* init = new triangulation(-1, tem,this);
+  init->iter = iter++;
+  allEdg2Tri = init->ed2tri;
   toSplit.push(init);
-  if((toSplit.top())->genus()!=0 || (toSplit.top())->aspectRatio() > eta ||
-				       (toSplit.top())->seamPoint){
+  //  printf("%12.5E\n",(toSplit.top())->aspectRatio() );
+  if((toSplit.top())->genus()!=0 ||
+     (toSplit.top())->aspectRatio() > eta ||
+     (toSplit.top())->seamPoint){
+
     while( !toSplit.empty()){
       std::vector<triangulation*> part;
       triangulation* tosplit = toSplit.top();
@@ -142,6 +230,7 @@ void discreteFace::createGeometry()
       delete tosplit;
 
       for(unsigned int i=0; i<part.size(); i++){
+	part[i]->iter = iter++;
 	if(part[i]->genus()!=0 || part[i]->aspectRatio() > eta || part[i]->seamPoint)
 	  toSplit.push(part[i]);
 	else{
@@ -157,25 +246,49 @@ void discreteFace::createGeometry()
   }
   updateTopology(toParam);
 
+
+  /*
+  FILE* debug = Fopen("tralala-init.pos","w");
+  fprintf(debug,"View \"discreteEdges\"{\n");
+  for(unsigned int j=0; j<toParam.size(); j++){
+
+    std::list<GEdge*> ge = toParam[j]->my_GEdges;
+    for(std::list<GEdge*>::iterator it = ge.begin(); it!=ge.end(); ++it){
+      if((*it)->tag()==112 ){
+
+	std::vector<MLine*> ml = (*it)->lines;
+	printf("___(init) map %d, ge %d___ (%p)\n",j+1,(*it)->tag(),(*it));
+	for(unsigned int i=0; i<ml.size(); i++){
+	  ml[i]->writePOS(debug,true,false,false,false,false,false,1.,(*it)->tag());
+	  printf("%d[%d;%d]--",ml[i]->getNum(),ml[i]->getVertex(0)->getNum(),ml[i]->getVertex(1)->getNum());
+	}
+	printf("\n");
+
+      }
+
+    }
+  }
+  fprintf(debug,"};");
+  fclose(debug);
+  */
+
   for(unsigned int i=0; i<toParam.size(); i++){
-    /*
-    char name[256];
-    sprintf(name,"map%d.pos",i);
-    toParam[i]->print(name,i);
-    */
+
     fillHoles(toParam[i]);
     //sprintf(name,"mapFilled%d.pos",i);
     //toParam[i]->print(name, toParam[i]->idNum);
   }
 
+
   for(unsigned int i=0; i<toParam.size(); i++){
     discreteDiskFace *df = new discreteDiskFace (this,toParam[i], order,(_CAD.empty() ? NULL : &_CAD));
-    //    df->printAtlasMesh();
-    //    df->putOnView(tag(), i, true,true);
+    df->printAtlasMesh();
+    df->printParamMesh();
+    //df->putOnView(tag(), i, true,true);
     df->replaceEdges(toParam[i]->my_GEdges);
     _atlas.push_back(df);
   }
-
+  complex_crossField();
 #endif
 }
 
@@ -228,9 +341,32 @@ void discreteFace::gatherMeshes()
 void discreteFace::mesh(bool verbose)
 {
 #if defined(HAVE_ANN) && defined(HAVE_SOLVER) && defined(HAVE_MESH)
+  for(unsigned int j=0; j<_atlas.size(); j++){
+
+    std::list<GEdge*> ge = _atlas[j]->edges();
+    for(std::list<GEdge*>::iterator it = ge.begin(); it!=ge.end(); ++it){
+
+      char myname[256];
+      sprintf(myname,"tralala%d_%d.pos",(*it)->tag(),j+1);
+      FILE* debug = Fopen(myname,"w");
+      fprintf(debug,"View \"discreteEdges\"{\n");
+      std::vector<MLine*> ml = (*it)->lines;
+
+      //printf("___map %d (%d), ge %d___ (%p)\n",j+1,_atlas[j]->tag(),(*it)->tag(),(*it));
+      for(unsigned int i=0; i<ml.size(); i++){
+	ml[i]->writePOS(debug,false,true,false,false,false,false,1.,(*it)->tag());
+	//printf("%d[%d;%d]--",ml[i]->getNum(),ml[i]->getVertex(0)->getNum(),ml[i]->getVertex(1)->getNum());
+      }
+      //printf("\n");
+
+      fprintf(debug,"};");
+      fclose(debug);
+    }
+  }
+
   if (!CTX::instance()->meshDiscrete) return;
 
-  Msg::Info("Discrete Face %d is going to be meshed",tag());
+  //Msg::Info("Discrete Face %d is going to be meshed",tag());
   for (unsigned int i=0;i<_atlas.size();i++){
     //    {
       //      void openProblemsON(void);
@@ -365,10 +501,13 @@ void discreteFace::setupDiscreteEdge(discreteEdge*de,std::vector<MLine*>mlines,
 // split old GEdge's
 void discreteFace::splitDiscreteEdge(GEdge *de , GVertex *gv, discreteEdge* newE[2])
 {
+
   MVertex *vend = de->getEndVertex()->mesh_vertices[0];
 
-  newE[0] = new discreteEdge (de->model(),model()->getMaxElementaryNumber(1) + 1,de->getBeginVertex(),gv);
-  newE[1] = new discreteEdge (de->model(),model()->getMaxElementaryNumber(1) + 1,gv, de->getEndVertex());
+  int mytag = this->model()->getMaxElementaryNumber(1) + 1;
+  newE[0] = new discreteEdge (de->model(),mytag,de->getBeginVertex(),gv);
+  mytag++;
+  newE[1] = new discreteEdge (de->model(),mytag,gv, de->getEndVertex());
 
   int current = 0;
   std::vector<MLine*> mlines;
@@ -405,15 +544,17 @@ void discreteFace::splitDiscreteEdge(GEdge *de , GVertex *gv, discreteEdge* newE
 	  else tagEdges.push_back (geit2->tag());
 	}
 	df->l_edges.clear();
-	df->setBoundEdges (df->model(), tagEdges);
+	df->setBoundEdges(tagEdges);
+      }
+      else{
+        Msg::Error("splitDiscreteEdge only applies to discrete geometries");
+        return;
       }
-      else Msg::Fatal("discreteFace::splitDiscreteEdge, This only applies to discrete geometries");
     }
   }
   de->model()->remove(de);
 }
 
-
 void discreteFace::split(triangulation* trian,std::vector<triangulation*> &partition,
                          int nPartitions)
 {
@@ -488,7 +629,7 @@ void discreteFace::split(triangulation* trian,std::vector<triangulation*> &parti
       }// end for j
     }// end while
     if(!celem.empty()){// if the set is empty, it means that the part was connected
-      Msg::Info("discreteFace::split(), a partition (%d) is not connected: it is going to be fixed.",p);
+      //Msg::Info("discreteFace::split(), a partition (%d) is not connected: it is going to be fixed.",p);
       std::vector<MElement*> relem(celem.begin(),celem.end());// new part
       for(unsigned int ie=0; ie<relem.size(); ie++)// updating the id part of the element belonging to the new part
 	el2part[relem[ie]] = nPartitions;
@@ -584,7 +725,7 @@ void discreteFace::updateTopology(std::vector<triangulation*>&partition)
 	  }// end else oe==end()
 	}// end imvt
 	// the new GEdge can be created with its GVertex
-	discreteEdge* internalE = new discreteEdge (this->model(),model()->getMaxElementaryNumber(1) + 1,v[0],v[1]);
+	discreteEdge* internalE = new discreteEdge (this->model(),this->model()->getMaxElementaryNumber(1) + 1,v[0],v[1]);
 	setupDiscreteEdge(internalE,myLines,&todelete);
 	gGEdges.insert(internalE);
 
@@ -603,7 +744,7 @@ void discreteFace::updateTopology(std::vector<triangulation*>&partition)
 	}
 	v = new discreteVertex (this->model(),model()->getMaxElementaryNumber(0) + 1);
 	setupDiscreteVertex(v,cv0,&todelete);
-	discreteEdge* gloop = new discreteEdge (this->model(),model()->getMaxElementaryNumber(1) + 1,v,v);
+	discreteEdge* gloop = new discreteEdge (this->model(),this->model()->getMaxElementaryNumber(1) + 1,v,v);
 	setupDiscreteEdge(gloop,my_MLines,&todelete);
 	gGEdges.insert(gloop);
       }// end while v02edg.empty()
@@ -708,6 +849,299 @@ void discreteFace::addTriangle(triangulation* trian, MTriangle* t)
 #endif
 }
 
+
+
+void discreteFace::complex_crossField()
+{
+#if defined(HAVE_SOLVER) && defined(HAVE_ANN)
+  // COMPLEX linear system
+  linearSystem<std::complex<double> > * lsys;
+
+#ifdef HAVE_PETSC
+#ifdef PETSC_USE_COMPLEX
+  lsys = new linearSystemPETSc<std::complex<double> >;
+#else
+  Msg::Error("Petsc complex is required (we do need complex in discreteFace::complex_crossField())");
+  return;
+#endif
+#else
+  Msg::Error("Petsc is required (we do need complex in discreteFace::crossField())");
+  return;
+#endif
+
+  std::complex<double> i1(0,1);
+  dofManager<std::complex<double> > myAssembler(lsys);
+
+  std::map<MEdge,int,Less_Edge> ed2key;
+  for (unsigned int i=0; i<triangles.size(); i++){
+
+    MTriangle* tri = triangles[i];
+    for(int j=0; j<3; j++){
+
+      MEdge ed = tri->getEdge(j);
+      std::vector<int> iTri = allEdg2Tri[ed];
+      int mini = iTri[0];
+      if (iTri.size()>1)
+	mini = iTri[0] < iTri[1] ? iTri[0] : iTri[1];
+      else
+	myAssembler.fixDof(3*mini+j,0,std::complex<double>(1)); // #tocheck
+
+      int num,s;
+      triangles[mini]->getEdgeInfo(ed,num,s);
+      myAssembler.numberDof(3*mini+num,0);
+      ed2key[ed] = 3*mini+num;
+    }// end for j
+  }// end for unsigned int i
+
+
+  double grad[3][3] = {{0.,-2.,0.},{2.,2.,0.},{-2.,0.,0.}};
+  for (unsigned int i=0; i<triangles.size(); i++){
+
+    MTriangle* tri = triangles[i];
+
+    double jac[3][3];
+    double invjac[3][3];
+    double dJac = tri->getJacobian(0., 0., 0., jac);
+    inv3x3(jac, invjac);
+
+    for(int j=0; j<3; j++){
+
+      double alpha_j = getAlpha(tri,j);
+
+      std::complex<double> ej(std::exp(4.*i1*alpha_j));
+      Dof R(ed2key[tri->getEdge(j)],0);
+      for(int k=0; k<3; k++){
+
+	double alpha_k = getAlpha(tri,k);
+
+	std::complex<double> ek(std::exp(-4.*i1*alpha_k));
+	std::complex<double> K_jk = 0.;
+	for (int l=0; l<3; l++)
+	  for(int jj=0; jj<3; jj++)
+	    for(int kk=0; kk<3; kk++)
+	      K_jk += grad[j][jj] * invjac[l][jj] * invjac[l][kk] * grad[k][kk];
+	K_jk *= ej*ek*dJac/2.;
+
+	Dof C(ed2key[tri->getEdge(k)],0);
+	myAssembler.assemble(R,C,K_jk);
+      }// end for k
+
+    }// end for j
+
+  }// end for unsigned int
+
+
+  lsys->systemSolve();
+  FILE* myfile = Fopen("crossField.pos","w");
+  fprintf(myfile,"View \"cross\"{\n");
+  for(unsigned int i=0; i<triangles.size(); i++){
+    fprintf(myfile,"VT(");
+    MTriangle* tri = triangles[i];
+    MEdge ed = tri->getEdge(0);
+
+    MVertex *v0, *v1, *v2;
+
+    v0 = tri->getVertex(0);
+    v1 = tri->getVertex(1);
+    v2 = tri->getVertex(2);
+
+    SVector3 a(v1->x()-v0->x(),v1->y()-v0->y(),v1->z()-v0->z());
+    SVector3 b(v2->x()-v0->x(),v2->y()-v0->y(),v2->z()-v0->z());
+    SVector3 n = crossprod(a,b); n.normalize();
+
+    v0 = ed.getSortedVertex(0);
+    v1 = ed.getSortedVertex(1);
+    SVector3 e(v1->x()-v0->x(),v1->y()-v0->y(),v1->z()-v0->z());
+    e.normalize();
+    SVector3 d = crossprod(n,e); d.normalize();
+
+    std::vector<double> U; U.resize(3);
+    std::vector<double> V; V.resize(3);
+    for(int j=0; j<3; j++){
+      fprintf(myfile,"%f,%f,%f",tri->getVertex(j)->x(),tri->getVertex(j)->y(),tri->getVertex(j)->z());
+      if (j<2) fprintf(myfile,",");
+      MEdge ed = tri->getEdge(j);
+      std::complex<double> fstar; // edge basis
+      myAssembler.getDofValue(ed2key[ed],0,fstar); // conjugate dof in the local edge basis
+      double alpha = getAlpha(tri,j); // triangle basis
+      std::complex<double> F(std::exp(4.*i1*alpha));
+      F *= std::conj(fstar); // dof of the local tri basis
+      U[j] = std::real(F);
+      V[j] = std::imag(F);
+    }
+    fprintf(myfile,")");
+    std::vector<double> Fu, Fv;
+    crouzeixRaviart(U,Fu);
+    crouzeixRaviart(V,Fv);
+    fprintf(myfile,"{");
+    for(int j=0; j<3; j++){
+      double u = Fu[j], v = Fv[j]; double theta = std::atan2(v,u)/4.;
+      SVector3 cf(cos(theta)*e.x()+sin(theta)*d.x(),cos(theta)*e.y()+sin(theta)*d.y(),cos(theta)*e.z()+sin(theta)*d.z());
+      cf = cf*sqrt(u*u+v*v);
+      fprintf(myfile,"%f,%f,%f",cf.x(),cf.y(),cf.z());
+      if (j<2) fprintf(myfile,",");
+    }
+    fprintf(myfile,"};\n");
+  }
+  fprintf(myfile,"};");
+  fclose(myfile);
+#endif
+}
+
+
+
+
+void discreteFace::crossField()
+{
+#if defined(HAVE_SOLVER) && defined(HAVE_ANN)
+  // linear system
+  linearSystem<double> * lsys = 0;
+
+#ifdef HAVE_PETSC
+  lsys = new linearSystemPETSc<double>;
+#else
+  Msg::Error("Petsc is required for cross fields");
+  return;
+#endif
+
+  dofManager<double> myAssembler(lsys);
+
+  std::map<MEdge,int,Less_Edge> ed2key;
+  for (unsigned int i=0; i<triangles.size(); i++){
+
+    MTriangle* tri = triangles[i];
+    for(int j=0; j<3; j++){
+
+      MEdge ed = tri->getEdge(j);
+      std::vector<int> iTri = allEdg2Tri[ed];
+      int mini = iTri[0];
+      if (iTri.size()>1)
+	mini = iTri[0] < iTri[1] ? iTri[0] : iTri[1];
+      else{
+	myAssembler.fixDof(3*mini+j,0,1.); // setting theta and not Theta
+	myAssembler.fixDof(3*mini+j,1,0.);
+      }
+
+      int num,s;
+      triangles[mini]->getEdgeInfo(ed,num,s);
+      myAssembler.numberDof(3*mini+num,0); // u, not U
+      myAssembler.numberDof(3*mini+num,1); // v, not V
+      ed2key[ed] = 3*mini+num;
+    }// end for j
+  }// end for unsigned int i
+
+  double grad[3][3] = {{0.,-2.,0.},{2.,2.,0.},{-2.,0.,0.}};
+  for (unsigned int i=0; i<triangles.size(); i++){
+
+    MTriangle* tri = triangles[i];
+
+    double jac[3][3];
+    double invjac[3][3];
+    double dJac = tri->getJacobian(0., 0., 0., jac);
+    inv3x3(jac, invjac);
+
+    for(int j=0; j<3; j++){
+
+      double alpha_j = getAlpha(tri,j);
+
+      Dof Ru(ed2key[tri->getEdge(j)],0);
+      Dof Rv(ed2key[tri->getEdge(j)],1);
+      for(int k=0; k<3; k++){
+
+	double alpha_k = getAlpha(tri,k);
+
+	Dof Cu(ed2key[tri->getEdge(k)],0);
+	Dof Cv(ed2key[tri->getEdge(k)],1);
+
+	double K_jk = 0.;
+	for (int l=0; l<3; l++)
+	  for(int jj=0; jj<3; jj++)
+	    for(int kk=0; kk<3; kk++)
+	      K_jk += grad[j][jj] * invjac[l][jj] * invjac[l][kk] * grad[k][kk];
+	K_jk *= dJac/2.;
+	//printf("%f\t",K_jk);
+	//printf("%f \t %f \n %f \t %f \n",cos(4.*(alpha_j-alpha_k))*K_jk,sin(4.*(alpha_j-alpha_k))*K_jk,sin(4.*(alpha_k-alpha_j))*K_jk,cos(4.*(alpha_j-alpha_k))*K_jk);
+
+
+	myAssembler.assemble(Ru,Cu,cos(4.*(alpha_j-alpha_k))*K_jk);
+	myAssembler.assemble(Ru,Cv,sin(4.*(alpha_j-alpha_k))*K_jk);
+	myAssembler.assemble(Rv,Cu,sin(4.*(alpha_k-alpha_j))*K_jk);
+	myAssembler.assemble(Rv,Cv,cos(4.*(alpha_j-alpha_k))*K_jk);
+      }// end for k
+      //printf("\n");
+
+    }// end for j
+    //printf("\n");
+  }// end for unsigned int
+
+
+  lsys->systemSolve();
+  FILE* myfile = Fopen("crossField.pos","w");
+  fprintf(myfile,"View \"cross\"{\n");
+  for(unsigned int i=0; i<triangles.size(); i++){
+    fprintf(myfile,"VT(");
+    MTriangle* tri = triangles[i];
+    MEdge ed = tri->getEdge(0);
+
+    MVertex *v0, *v1, *v2;
+
+    v0 = tri->getVertex(0);
+    v1 = tri->getVertex(1);
+    v2 = tri->getVertex(2);
+
+    SVector3 a(v1->x()-v0->x(),v1->y()-v0->y(),v1->z()-v0->z());
+    SVector3 b(v2->x()-v0->x(),v2->y()-v0->y(),v2->z()-v0->z());
+    SVector3 n = crossprod(a,b); n.normalize();
+
+    v0 = ed.getSortedVertex(0);
+    v1 = ed.getSortedVertex(1);
+    SVector3 e(v1->x()-v0->x(),v1->y()-v0->y(),v1->z()-v0->z());
+    e.normalize();
+    //printf("e=(%f %f)\n",e.x(),e.y());
+    SVector3 d = crossprod(n,e); d.normalize();
+    //printf("d=(%f %f)\n",d.x(),d.y());
+
+
+    std::vector<double> U; U.resize(3);
+    std::vector<double> V; V.resize(3);
+    for(int j=0; j<3; j++){
+      fprintf(myfile,"%f,%f,%f",tri->getVertex(j)->x(),tri->getVertex(j)->y(),tri->getVertex(j)->z());
+      if (j<2) fprintf(myfile,",");
+      MEdge ed = tri->getEdge(j);
+      double  u, v;// edge basis
+      myAssembler.getDofValue(ed2key[ed],0,u);// u
+      myAssembler.getDofValue(ed2key[ed],1,v);// v
+      if(std::abs(u*u+v*v-1)>1e-12)
+	printf("/!\\ ---> warning unit vector \t %f  \n",sqrt(u*u+v*v));
+      double alpha = getAlpha(tri,j);// triangle basis
+      U[j] = cos(4.*alpha)*u - sin(4.*alpha)*v;// U, not u
+      V[j] = sin(4.*alpha)*u + cos(4.*alpha)*v;// V, not v
+      //printf("sol: ed%d[%f,%f]--[%f,%f] \t Theta = %f (tri) ~ theta = %f (ed) ~ alpha = %f ~u=%f, v=%f VS U=%f, V=%f \n",j,ed.getSortedVertex(0)->x(),ed.getSortedVertex(0)->y(),ed.getSortedVertex(1)->x(),ed.getSortedVertex(1)->y(),std::atan2(V[j],U[j])*180./(4.*M_PI),std::atan2(v,u)*180./(4.*M_PI),alpha*180./M_PI,u,v,U[j],V[j]);
+    }
+    fprintf(myfile,")");
+    std::vector<double> Fu, Fv;
+    crouzeixRaviart(U,Fu);
+    crouzeixRaviart(V,Fv);
+    fprintf(myfile,"{");
+    for(int j=0; j<3; j++){
+      double u = Fu[j], v = Fv[j]; double theta = std::atan2(v,u)/4.;
+      //printf("theta_%d = %f\n",j,theta*180./M_PI);
+      SVector3 cf(cos(theta)*e.x()+sin(theta)*d.x(),cos(theta)*e.y()+sin(theta)*d.y(),cos(theta)*e.z()+sin(theta)*d.z());
+      //cf = cf*sqrt(u*u+v*v);
+      fprintf(myfile,"%f,%f,%f",cf.x(),cf.y(),cf.z());//cos(theta)*e.x()-sin(theta)*e.y(),sin(theta)*e.x()+cos(theta)*e.y(),e.z());
+
+      if( std::abs(dot(cf,n)) > 1e-12)
+	printf("/!\\ ---> warning orthogonality \t %f \n",dot(cf,n));
+      if (j<2) fprintf(myfile,",");
+    }
+    fprintf(myfile,"};\n");
+  }
+  fprintf(myfile,"};");
+  fclose(myfile);
+#endif
+}
+
+
 void discreteFace::writeGEO(FILE *fp)
 {
   fprintf(fp, "Discrete Face(%d) = {",tag());
