@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -6,6 +6,7 @@
 #include "GmshConfig.h"
 #include "discreteRegion.h"
 #include "MVertex.h"
+#include "GModelIO_GEO.h"
 #include "Geo.h"
 #include "Context.h"
 
@@ -16,16 +17,43 @@
 
 discreteRegion::discreteRegion(GModel *model, int num) : GRegion(model, num)
 {
-  ::Volume *v = Create_Volume(num, MSH_VOLUME_DISCRETE);
+  ::Volume *v = CreateVolume(num, MSH_VOLUME_DISCRETE);
   Tree_Add(model->getGEOInternals()->Volumes, &v);
 }
 
-void discreteRegion::setBoundFaces(std::set<int> tagFaces)
+void discreteRegion::setBoundFaces(const std::set<int> &tagFaces)
 {
-  for (std::set<int>::iterator it = tagFaces.begin() ; it != tagFaces.end();++it){
+  for (std::set<int>::const_iterator it = tagFaces.begin() ; it != tagFaces.end();++it){
     GFace *face = model()->getFaceByTag(*it);
-    l_faces.push_back(face);
-    face->addRegion(this);
+    if(face){
+      l_faces.push_back(face);
+      face->addRegion(this);
+    }
+    else{
+      Msg::Error("Unknown model face %d", *it);
+    }
+  }
+}
+
+void discreteRegion::setBoundFaces(const std::vector<int> &tagFaces,
+                                   const std::vector<int> &signFaces)
+{
+  if(tagFaces.size() != signFaces.size()){
+    Msg::Error("Wrong number of face signs in setBoundFaces");
+    std::set<int> tags;
+    tags.insert(tagFaces.begin(), tagFaces.end());
+    setBoundFaces(tags);
+  }
+  for (unsigned int i = 0; i != tagFaces.size(); i++){
+    GFace *face = model()->getFaceByTag(tagFaces[i]);
+    if(face){
+      l_faces.push_back(face);
+      face->addRegion(this);
+      l_dirs.push_back(signFaces[i]);
+    }
+    else{
+      Msg::Error("Unknown model face %d", tagFaces[i]);
+    }
   }
 }
 
