@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -15,7 +15,9 @@ class discreteRegion : public GRegion {
   discreteRegion(GModel *model, int num);
   virtual ~discreteRegion() {}
   virtual GeomType geomType() const { return DiscreteVolume; }
-  void setBoundFaces(std::set<int> tagFaces);
+  void setBoundFaces(const std::set<int> &tagFaces);
+  void setBoundFaces(const std::vector<int> &tagFaces,
+                     const std::vector<int> &signFaces);
   void findFaces(std::map<MFace, std::vector<int>, Less_Face> &map_faces);
   virtual void remesh();
 };
