@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -20,6 +20,14 @@ gmshEdge::gmshEdge(GModel *m, Curve *edge, GVertex *v1, GVertex *v2)
   resetMeshAttributes();
 }
 
+void gmshEdge::resetNativePtr(Curve *edge, GVertex *_v1, GVertex *_v2)
+{
+  c = edge;
+  v0 = _v1; v1 = _v2;
+  if(v0) v0->addEdge(this);
+  if(v1 && v1 != v0) v1->addEdge(this);
+}
+
 bool gmshEdge::degenerate(int dim) const
 {
   if (c->beg == c->end &&
