@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -6,12 +6,13 @@
 #ifndef _GMSH_EDGE_H_
 #define _GMSH_EDGE_H_
 
-#include "Geo.h"
 #include "GEdge.h"
 
+class Curve;
+
 class gmshEdge : public GEdge {
  protected:
-  Curve *c; 
+  Curve *c;
 
  public:
   gmshEdge(GModel *model, Curve *edge, GVertex *v1, GVertex *v2);
@@ -22,7 +23,7 @@ class gmshEdge : public GEdge {
   virtual SVector3 firstDer(double par) const;
   virtual SVector3 secondDer(double par) const;
   ModelType getNativeType() const { return GmshModel; }
-  void * getNativePtr() const { return c; }
+  void *getNativePtr() const { return c; }
   virtual std::string getAdditionalInfoString();
   virtual int minimumMeshSegments() const;
   virtual int minimumDrawSegments() const;
@@ -31,6 +32,7 @@ class gmshEdge : public GEdge {
   virtual void writeGEO(FILE *fp);
   void discretize(double tol, std::vector<SPoint3> &dpts, std::vector<double> &ts);
   virtual bool degenerate(int dim) const;
+  void resetNativePtr(Curve *edge, GVertex *v1, GVertex *v2);
 };
 
 #endif
