@@ -21,7 +21,7 @@
 #endif
 
 gmshFace::gmshFace(GModel *m, Surface *face)
-  : GFace(m, face->Num), isSphere(false), radius(0.)
+  : GFace(m, face->Num)
 {
   resetNativePtr(face);
   resetMeshAttributes();
@@ -105,7 +105,6 @@ void gmshFace::resetNativePtr(Surface *face)
   // the bounding vertices)
   if(s->Typ == MSH_SURF_PLAN) computeMeanPlane();
 
-  isSphere = IsRuledSurfaceASphere(s, center, radius);
 }
 
 double gmshFace::getMetricEigenvalue(const SPoint2 &pt)
