@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -22,6 +22,27 @@
 #include "ANN/ANN.h"
 #endif
 
+std::set<gLevelset*, gLevelsetLessThan> gLevelset::all_;
+int gLevelset::maxTag_ = 0;
+
+bool gLevelsetLessThan::operator()(const gLevelset *l1, const gLevelset *l2) const
+{
+  return l1->getTag() < l2->getTag();
+}
+
+gLevelset *gLevelset::find(int tag)
+{
+  gLevelset l(tag);
+  std::set<gLevelset*, gLevelsetLessThan>::iterator it = all_.find(&l);
+  if(it == all_.end()) return 0;
+  return *it;
+}
+
+void gLevelset::add(gLevelset *l)
+{
+  all_.insert(l);
+}
+
 void insertActiveCells(double x, double y, double z, double rmax,
                        cartesianBox<double> &box)
 {
