@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -10,14 +10,23 @@
 #include "GmshMessage.h"
 
 gmshRegion::gmshRegion(GModel *m, ::Volume *volume)
-  : GRegion(m, volume->Num), v(volume)
+  : GRegion(m, volume->Num)
 {
+  resetNativePtr(volume);
+  resetMeshAttributes();
+}
+
+void gmshRegion::resetNativePtr(::Volume *volume)
+{
+  v = volume;
+  l_faces.clear();
+  l_dirs.clear();
   for(int i = 0; i < List_Nbr(v->Surfaces); i++){
     Surface *s;
     List_Read(v->Surfaces, i, &s);
     int ori;
     List_Read(v->SurfacesOrientations, i, &ori);
-    GFace *f = m->getFaceByTag(abs(s->Num));
+    GFace *f = model()->getFaceByTag(abs(s->Num));
     if(f){
       l_faces.push_back(f);
       l_dirs.push_back(ori);
@@ -29,7 +38,7 @@ gmshRegion::gmshRegion(GModel *m, ::Volume *volume)
   for(int i = 0; i < List_Nbr(v->SurfacesByTag); i++){
     int is;
     List_Read(v->SurfacesByTag, i, &is);
-    GFace *f = m->getFaceByTag(abs(is));
+    GFace *f = model()->getFaceByTag(abs(is));
     if(f){
       l_faces.push_back(f);
       l_dirs.push_back(gmsh_sign(is));
@@ -38,18 +47,6 @@ gmshRegion::gmshRegion(GModel *m, ::Volume *volume)
     else
       Msg::Error("Unknown surface %d", is);
   }
-  if(v->EmbeddedSurfaces){
-    for(int i = 0; i < List_Nbr(v->EmbeddedSurfaces); i++){
-      Surface *s;
-      List_Read(v->EmbeddedSurfaces, i, &s);
-      GFace *gf = m->getFaceByTag(abs(s->Num));
-      if(gf)
-	addEmbeddedFace(gf);
-      else
-	Msg::Error("Unknown surface %d", s->Num);
-    }
-  }
-  resetMeshAttributes();
 }
 
 void gmshRegion::resetMeshAttributes()
