@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -6,9 +6,10 @@
 #ifndef _GMSH_REGION_H_
 #define _GMSH_REGION_H_
 
-#include "Geo.h"
 #include "GRegion.h"
 
+class Volume;
+
 class gmshRegion : public GRegion {
  protected:
   ::Volume *v;
@@ -17,8 +18,9 @@ class gmshRegion : public GRegion {
   virtual ~gmshRegion() {}
   virtual GeomType geomType() const;
   ModelType getNativeType() const { return GmshModel; }
-  void * getNativePtr() const { return v; }
+  void *getNativePtr() const { return v; }
   virtual void resetMeshAttributes();
+  void resetNativePtr(::Volume *_v);
 };
 
 #endif
