@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -33,6 +33,12 @@
 #endif
 
 drawContextGlobal *drawContext::_global = 0;
+void (*drawContext::drawGeomTransient)(void*) = 0;
+
+void drawContext::setDrawGeomTransientFunction(void (*fct)(void *))
+{
+  drawGeomTransient = fct;
+}
 
 extern SPoint2 getGraph2dDataPointForTag(unsigned int);
 
@@ -860,7 +866,7 @@ static MElement *getElement(GEntity *e, int va_type, int index)
   return 0;
 }
 
-bool drawContext::select(int type, bool multiple, bool mesh,
+bool drawContext::select(int type, bool multiple, bool mesh, bool post,
                          int x, int y, int w, int h,
                          std::vector<GVertex*> &vertices,
                          std::vector<GEdge*> &edges,
@@ -903,7 +909,7 @@ bool drawContext::select(int type, bool multiple, bool mesh,
   initPosition();
   drawGeom();
   if(mesh) drawMesh();
-  drawPost();
+  if(post) drawPost();
   drawGraph2d(true);
 
   // 2d stuff
