@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -172,17 +172,51 @@ static void drawScaleLabel(drawContext *ctx, PView *p, double xmin, double ymin,
   char label[1024];
 
   int nt = data->getNumTimeSteps();
-  if((opt->showTime == 1 && nt > 1) || opt->showTime == 2){
-    char tmp[256];
-    sprintf(tmp, opt->format.c_str(), data->getTime(opt->timeStep));
-    sprintf(label, "%s - Time: %s", data->getName().c_str(), tmp);
+  int n0 = data->getFirstNonEmptyTimeStep();
+  int n = (nt - n0 > 0) ? nt - n0 : 1;
+  char time[256];
+  sprintf(time, opt->format.c_str(), data->getTime(opt->timeStep));
+  int choice = opt->showTime;
+  if(choice == 3){ // automatic
+    if(n == 1) choice = 0; // nothing
+    else if(n == 2) choice = 2; // harmonic
+    else choice = 5; // multi-step data
   }
-  else if((opt->showTime == 3 && nt > 1) || opt->showTime == 4){
-    sprintf(label, "%s - Step %d in [0,%d]", data->getName().c_str(), opt->timeStep,
-            data->getNumTimeSteps()-1);
-  }
-  else
+  switch(choice){
+  case 1: // time series
+    sprintf(label, "%s - time %s", data->getName().c_str(), time);
+    break;
+  case 2: // harmonic data
+    if(n <= 2)
+      sprintf(label, "%s - %s part", data->getName().c_str(),
+              ((opt->timeStep - n0) % 2) ? "imaginary" : "real");
+    else
+      sprintf(label, "%s - harmonic %s (%s part)",
+              data->getName().c_str(), time,
+              ((opt->timeStep - n0) % 2) ? "imaginary" : "real");
+    break;
+  case 3: // automatic
+    // never here
+    break;
+  case 4: // step data
+    sprintf(label, "%s - step %d", data->getName().c_str(), opt->timeStep);
+    break;
+  case 5: // multi-step data
+    sprintf(label, "%s - step %d in [0,%d]", data->getName().c_str(),
+            opt->timeStep, data->getNumTimeSteps() - 1);
+    break;
+  case 6: // real eigenvalues
+    sprintf(label, "%s - eigenvalue %s", data->getName().c_str(),
+            time);
+    break;
+  case 7: // complex eigenvalues
+    sprintf(label, "%s - eigenvalue %s (%s part)", data->getName().c_str(),
+            time, ((opt->timeStep - n0) % 2) ? "imaginary" : "real");
+    break;
+  default:
     sprintf(label, "%s", data->getName().c_str());
+    break;
+  }
 
   if(horizontal){
     ctx->drawString(label, xmin + width / 2., ymin + height + tic + 1.4 * font_h, 0.,
