@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -7,6 +7,8 @@
 //   Koen Hillewaert
 //
 
+#include <sstream>
+#include "GmshConfig.h"
 #include "HighOrder.h"
 #include "MLine.h"
 #include "MTriangle.h"
@@ -23,7 +25,9 @@
 #include "BasisFactory.h"
 #include "MVertexRTree.h"
 
-#include <sstream>
+#if defined(HAVE_OPTHOM)
+#include "OptHomFastCurving.h"
+#endif
 
 // --------- Functions that help optimizing placement of points on geometry -----------
 
@@ -132,13 +136,14 @@ static bool computeEquidistantParameters(GFace *gf, double u0, double uN,
 
 // --------- Creation of high-order edge vertices -----------
 
-static bool getEdgeVerticesonGeo(GEdge *ge, MVertex *v0, MVertex *v1,
+static bool getEdgeVerticesOnGeo(GEdge *ge, MVertex *v0, MVertex *v1,
                                  std::vector<MVertex*> &ve, int nPts = 1)
 {
   static const double relaxFail = 1e-2;
   double u0 = 0., u1 = 0., US[100];
   bool reparamOK = reparamMeshVertexOnEdge(v0, ge, u0);
-  if(ge->periodic(0) && ge->getEndVertex()->getNumMeshVertices() > 0 &&
+  if(ge->periodic(0) && ge->getEndVertex() &&
+     ge->getEndVertex()->getNumMeshVertices() > 0 &&
      v1 == ge->getEndVertex()->mesh_vertices[0])
     u1 = ge->parBounds(0).high();
   else
@@ -178,7 +183,7 @@ static bool getEdgeVerticesonGeo(GEdge *ge, MVertex *v0, MVertex *v1,
   return true;
 }
 
-static bool getEdgeVerticesonGeo(GFace *gf, MVertex *v0, MVertex *v1,
+static bool getEdgeVerticesOnGeo(GFace *gf, MVertex *v0, MVertex *v1,
                                  std::vector<MVertex*> &ve, int nPts = 1)
 {
   SPoint2 p0, p1;
@@ -244,7 +249,8 @@ static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
 {
   if(ge->geomType() == GEntity::DiscreteCurve ||
      ge->geomType() == GEntity::BoundaryLayerCurve ||
-     ge->geomType() == GEntity::CompoundCurve)
+     ge->geomType() == GEntity::CompoundCurve ||
+     ge->geomType() == GEntity::PartitionCurve)
     linear = true;
 
   std::vector<MVertex*> veOld;
@@ -255,7 +261,7 @@ static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
   std::vector<MVertex*> veEdge;
   // Get vertices on geometry if asked
   bool gotVertOnGeo = linear ? false :
-    getEdgeVerticesonGeo(ge, veOld[0], veOld[1], veEdge, nPts);
+    getEdgeVerticesOnGeo(ge, veOld[0], veOld[1], veEdge, nPts);
   // If not on geometry, create from mesh interpolation
   if (!gotVertOnGeo)
     interpVerticesInExistingEdge(ge, ele, veEdge, nPts);
@@ -279,7 +285,8 @@ static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
 {
   if(gf->geomType() == GEntity::DiscreteSurface ||
      gf->geomType() == GEntity::BoundaryLayerSurface ||
-     gf->geomType() == GEntity::CompoundSurface)
+     gf->geomType() == GEntity::CompoundSurface ||
+     gf->geomType() == GEntity::PartitionSurface)
     linear = true;
 
   for(int i = 0; i < ele->getNumEdges(); i++) {
@@ -298,7 +305,7 @@ static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
     else { // Vertices do not exist, create them
       // Get vertices on geometry if asked
       bool gotVertOnGeo = linear ? false :
-        getEdgeVerticesonGeo(gf, veOld[0], veOld[1], veEdge, nPts);
+        getEdgeVerticesOnGeo(gf, veOld[0], veOld[1], veEdge, nPts);
       if (!gotVertOnGeo) {
         // If not on geometry, create from mesh interpolation
         const MLineN edgeEl(veOld, ele->getPolynomialOrder());
@@ -412,38 +419,39 @@ static void reorientQuadPoints(std::vector<MVertex*> &vtcs, int orientation,
       for (int i = 0; i < 4; i++) vtcs[start + i] = tmp[i];
 
       // EDGES
+      start += 4;
       for (int iEdge=0;iEdge<4;iEdge++){
         int p1 = indices[iEdge];
         int p2 = indices[(iEdge+1)%4];
         int nbP = order-1;
         if      (p1 == 0 && p2 == 1){
-          for (int i = 4+0*nbP; i < 4+1*nbP; i++) tmp[index++] = vtcs[i];
+          for (int i = start+0*nbP; i < start+1*nbP; i++) tmp[index++] = vtcs[i];
         }
         else if (p1 == 1 && p2 == 2){
-          for (int i = 4+1*nbP; i< 4+2*nbP; i++) tmp[index++] = vtcs[i];
+          for (int i = start+1*nbP; i< start+2*nbP; i++) tmp[index++] = vtcs[i];
         }
         else if (p1 == 2 && p2 == 3){
-          for (int i = 4+2*nbP; i< 4+3*nbP; i++) tmp[index++] = vtcs[i];
+          for (int i = start+2*nbP; i< start+3*nbP; i++) tmp[index++] = vtcs[i];
         }
         else if (p1 == 3 && p2 == 0){
-          for (int i = 4+3*nbP; i< 4+4*nbP; i++) tmp[index++] = vtcs[i];
+          for (int i = start+3*nbP; i< start+4*nbP; i++) tmp[index++] = vtcs[i];
         }
         else if (p1 == 1 && p2 == 0){
-          for (int i = 4+1*nbP-1; i >= 4+0*nbP; i--) tmp[index++] = vtcs[i];
+          for (int i = start+1*nbP-1; i >= start+0*nbP; i--) tmp[index++] = vtcs[i];
         }
         else if (p1 == 2 && p2 == 1){
-          for (int i = 4+2*nbP-1; i >= 4+1*nbP; i--) tmp[index++] = vtcs[i];
+          for (int i = start+2*nbP-1; i >= start+1*nbP; i--) tmp[index++] = vtcs[i];
         }
         else if (p1 == 3 && p2 == 2){
-          for (int i = 4+3*nbP-1; i >= 4+2*nbP; i--) tmp[index++] = vtcs[i];
+          for (int i = start+3*nbP-1; i >= start+2*nbP; i--) tmp[index++] = vtcs[i];
         }
         else if (p1 == 0 && p2 == 3){
-          for (int i = 4+4*nbP-1; i >= 4+3*nbP; i--) tmp[index++] = vtcs[i];
+          for (int i = start+4*nbP-1; i >= start+3*nbP; i--) tmp[index++] = vtcs[i];
         }
         else Msg::Error("Something wrong in reorientQuadPoints");
       }
-      for (int i = 0; i < index; i++)vtcs[start+4+i] = tmp[i];
-      start += (4 + index);
+      for (int i = 0; i < index; i++)vtcs[start+i] = tmp[i];
+      start += index;
     }
 
     order -= 2;
@@ -512,7 +520,7 @@ static int getNewFacePointsInVolume(MElement *incomplete, int nPts,
   case TYPE_PYR:
     switch (nPts){
     case 0:
-    case 1: return -1;
+    case 1: points = BasisFactory::getNodalBasis(MSH_PYR_14)->points; break;
     case 2: points = BasisFactory::getNodalBasis(MSH_PYR_30)->points; break;
     case 3: points = BasisFactory::getNodalBasis(MSH_PYR_55)->points; break;
     case 4: points = BasisFactory::getNodalBasis(MSH_PYR_91)->points; break;
@@ -563,7 +571,7 @@ static void getFaceVerticesOnGeo(GFace *gf, MElement *incomplete, const MElement
       GPoint gp = gf->point(SPoint2(GUESS[0], GUESS[1]));
       // closest point is not necessary (slow and for high quality HO
       // meshes it should be optimized anyway afterwards + closest point
-      // is still buggy (e.g. BFGS for a plane Ruled Surface)
+      // is still buggy (e.g. BFGS for a plane "Ruled Surface")
       // GPoint gp = gf->closestPoint(SPoint3(X, Y, Z), GUESS);
       if (gp.g()){
         v = new MFaceVertex(gp.x(), gp.y(), gp.z(), gf, gp.u(), gp.v());
@@ -1220,10 +1228,10 @@ static MPyramid *setHighOrder(MPyramid *p, GRegion *gr,
 {
   std::vector<MVertex*> ve, vf, vr;
   getEdgeVertices(gr, p, ve, newHOVert, edgeVertices, linear, nPts);
-//  MPyramidN incpl(p->getVertex(0), p->getVertex(1), p->getVertex(2),
-//                  p->getVertex(3), p->getVertex(4), ve, nPts + 1, 0, p->getPartition());
-//  getFaceVertices(gr, &incpl, p, vf, faceVertices, edgeVertices, linear, nPts);
-  getFaceVertices(gr, 0, p, vf, newHOVert, faceVertices, edgeVertices, linear, nPts);
+  MPyramidN incpl(p->getVertex(0), p->getVertex(1), p->getVertex(2),
+                  p->getVertex(3), p->getVertex(4), ve, nPts + 1, 0, p->getPartition());
+  getFaceVertices(gr, &incpl, p, vf, newHOVert, faceVertices, edgeVertices, linear, nPts);
+  // getFaceVertices(gr, 0, p, vf, newHOVert, faceVertices, edgeVertices, linear, nPts);
   ve.insert(ve.end(), vf.begin(), vf.end());
   getVolumeVerticesPyramid(gr, p, ve, vr, newHOVert, linear, nPts);
   ve.insert(ve.end(), vr.begin(), vr.end());
@@ -1634,6 +1642,7 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete, bool onlyVisi
     Msg::ProgressMeter(++counter, nTot, false, msg);
     if (onlyVisible && !(*it)->getVisibility()) continue;
     setHighOrder(*it, newHOVert[*it], edgeVertices, faceVertices, linear, incomplete, nPts);
+    if ((*it)->getColumns() != 0) (*it)->getColumns()->clearElementData();
   }
 
   for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it) {
@@ -1641,6 +1650,7 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete, bool onlyVisi
     Msg::ProgressMeter(++counter, nTot, false, msg);
     if (onlyVisible && !(*it)->getVisibility())continue;
     setHighOrder(*it, newHOVert[*it], edgeVertices, faceVertices, linear, incomplete, nPts);
+    if ((*it)->getColumns() != 0) (*it)->getColumns()->clearElementData();
   }
 
   // Update all high order vertices
@@ -1651,6 +1661,19 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete, bool onlyVisi
   for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it)
     updateHighOrderVertices(*it, newHOVert[*it], onlyVisible);
 
+#if defined(HAVE_OPTHOM)
+  // Determine mesh dimension and curve BL elements
+  FastCurvingParameters p;
+  p.dim = 0;
+  for (GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it)
+    if ((*it)->getNumMeshElements() > 0) { p.dim = 3; break; }
+  if (p.dim == 0)
+    for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it)
+        if ((*it)->getNumMeshElements() > 0) { p.dim = 2; break; }
+  // if(p.dim > 0)
+  //   HighOrderMeshFastCurving(GModel::current(), p);
+#endif
+
   updatePeriodicEdgesAndFaces(m);
 
   double t2 = Cpu();
