@@ -1,30 +1,22 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <stdlib.h>
 #include <vector>
-// general
-#include "OS.h"
-#include "Geo.h"
-#include "Context.h"
-#include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "fullMatrix.h"
-// mesh
 #include "meshGFaceOptimize.h"
 #include "meshGRegionBoundaryRecovery.h"
 #include "meshGRegionDelaunayInsertion.h"
 #include "meshGRegionRelocateVertex.h"
 #include "delaunay3d.h"
-// model
 #include "GModel.h"
 #include "GRegion.h"
 #include "GFace.h"
 #include "discreteFace.h"
 #include "discreteRegion.h"
-// elements
 #include "MLine.h"
 #include "MTriangle.h"
 #include "MTetrahedron.h"
@@ -32,8 +24,8 @@
 #include "MPrism.h"
 #include "MHexahedron.h"
 
-// Recursive function to generate all combinations of 4 indices between start 
-// and end indices included 
+// Recursive function to generate all combinations of 4 indices between start
+// and end indices included
 // Jeanne - HEXTREME
 void combination_of_4( int combination[4], int start, int end, int index,
   const std::vector<MVertex*>& vertices, std::vector<MTetrahedron*>& tets)
@@ -55,17 +47,19 @@ void combination_of_4( int combination[4], int start, int end, int index,
 }
 
 // Fill a region with all possible tets genereated from the
-// combination of points assigned to it 
+// combination of points assigned to it
 // Jeanne - HEXTREME
-void create_all_possible_tets(GRegion* region, const std::vector<MVertex*>& vertices) {
+void create_all_possible_tets(GRegion* region, const std::vector<MVertex*>& vertices)
+{
   unsigned int nb_points = vertices.size();
 
   int combinaison[4];
   std::vector<MTetrahedron*> tets;
   combination_of_4(combinaison, 0, nb_points - 1, 0, vertices, tets);
-  std::cout << " Number of tets created - all possible combinations - " << tets.size() << std::endl;
+  std::cout << " Number of tets created - all possible combinations - "
+            << tets.size() << std::endl;
 
-  for (int i = 0; i < tets.size(); ++i) {
+  for (unsigned int i = 0; i < tets.size(); ++i) {
     region->addTetrahedron(tets[i]);
   }
 }
@@ -76,8 +70,8 @@ void create_all_possible_tets(GRegion* region, const std::vector<MVertex*>& vert
 GRegion * createDiscreteRegionFromRawData(GModel *gm, fullMatrix<double> &pts,
                                           fullMatrix<int> &triangles, bool all_tets)
 {
-  GRegion *gr = new discreteRegion(gm, NEWREG());
-  GFace *gf = new discreteFace(gm, NEWREG());
+  GRegion *gr = new discreteRegion(gm, gm->getMaxElementaryNumber(3) + 1);
+  GFace *gf = new discreteFace(gm, gm->getMaxElementaryNumber(2) + 1);
   gm->add(gr);
   gm->add(gf);
   std::list<GFace*> faces; faces.push_back(gf); gr->set(faces);
@@ -113,7 +107,8 @@ GRegion * createDiscreteRegionFromRawData(GModel *gm, fullMatrix<double> &pts,
     gf->triangles.push_back(t);
   }
 
-//  delaunayTriangulation(1, 1, vs, gr->tetrahedra); // tetrahedralization is done when recovering the boundary
+  // delaunayTriangulation(1, 1, vs, gr->tetrahedra);
+  // tetrahedralization is done when recovering the boundary
 
   // create all tets
   if (all_tets) {
