@@ -599,7 +599,7 @@ void BDS2GMSH(BDS_Mesh *m, GFace *gf,
       BDS_Point *p = *itp;
       if(recoverMap.find(p) == recoverMap.end()){
         MVertex *v = new MFaceVertex
-          (p->X, p->Y, p->Z, gf, m->scalingU * p->u, m->scalingV * p->v);
+          (p->X, p->Y, p->Z, gf, p->u, p->v);
         recoverMap[p] = v;
         gf->mesh_vertices.push_back(v);
       }
@@ -1007,7 +1007,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   }
 
   //  if (gf->degenerate(0))return 0;
-  
+
   // build a set with all points of the boundaries
   std::set<MVertex*, MVertexLessThanNum> all_vertices, boundary;
   std::list<GEdge*>::iterator ite = edges.begin();
@@ -1097,8 +1097,6 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   // Buid a BDS_Mesh structure that is convenient for doing the actual
   // meshing procedure
   BDS_Mesh *m = new BDS_Mesh;
-  m->scalingU = 1;
-  m->scalingV = 1;
 
   std::vector<BDS_Point*> points(all_vertices.size());
   SBoundingBox3d bbox;
@@ -1606,11 +1604,11 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   gf->triangles.insert(gf->triangles.begin(),blTris.begin(),blTris.end());
   gf->mesh_vertices.insert(gf->mesh_vertices.begin(),verts.begin(),verts.end());
 
-  
+
   if((CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine) &&
      !CTX::instance()->mesh.optimizeLloyd && !onlyInitialMesh && CTX::instance()->mesh.algoRecombine != 2)
     recombineIntoQuads(gf);
-  
+
 
 
   computeElementShapes(gf, gf->meshStatistics.worst_element_shape,
@@ -1858,8 +1856,8 @@ static bool buildConsecutiveListOfVertices(GFace *gf, GEdgeLoop &gel,
       if(pp == 0){
         double U, V;
         SPoint2 param = coords[i];
-        U = param.x() / m->scalingU;
-        V = param.y() / m->scalingV;
+        U = param.x();
+        V = param.y();
         pp = m->add_point(count + countTot, U, V, gf);
         if (ge->dim() == 0){
           pp->lcBGM() = BGM_MeshSize(ge, 0, 0, here->x(), here->y(), here->z());
@@ -1912,8 +1910,6 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
   // Buid a BDS_Mesh structure that is convenient for doing the actual
   // meshing procedure
   BDS_Mesh *m = new BDS_Mesh;
-  m->scalingU = 1;
-  m->scalingV = 1;
 
   std::vector<std::vector<BDS_Point*> > edgeLoops_BDS;
   SBoundingBox3d bbox;
@@ -1999,8 +1995,10 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
     // the domain, use negative number to distinguish those fake
     // vertices
 
-    // FIX A BUG HERE IF THE SIZE OF THE BOX IS ZERO
-    bbox.makeCube();
+    if(du / dv < 1200 && dv / du < 1200){
+      // FIX A BUG HERE IF THE SIZE OF THE BOX IS ZERO
+      bbox.makeCube();
+    }
 
     bbox *= 3.5;
     MVertex *bb[4];
@@ -2250,6 +2248,13 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
                                  gf->meshStatistics.nbGoodLength);*/
     gf->meshStatistics.status = GFace::DONE;
 
+
+    if(debug){
+      char name[245];
+      sprintf(name, "surface%d-just-real.pos", gf->tag());
+      outputScalarField(m->triangles, name, 0);
+    }
+
     //    if(CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine || 1) {
     //            backgroundMesh::unset();
     //    }
@@ -2297,7 +2302,7 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
     }
     // recoverMap.insert(new_relations.begin(), new_relations.end());
   }
-  Msg::Info("%d points that are duplicated for Delaunay meshing", equivalence.size());
+  //  Msg::Info("%d points that are duplicated for Delaunay meshing", equivalence.size());
 
   // fill the small gmsh structures
   {
@@ -2306,7 +2311,7 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
       BDS_Point *p = *itp;
       if(recoverMap.find(p) == recoverMap.end()){
         MVertex *v = new MFaceVertex
-          (p->X, p->Y, p->Z, gf, m->scalingU * p->u, m->scalingV * p->v);
+          (p->X, p->Y, p->Z, gf, p->u, p->v);
         recoverMap[p] = v;
         gf->mesh_vertices.push_back(v);
       }
@@ -2503,6 +2508,10 @@ void meshGFace::operator() (GFace *gf, bool print)
              gf->geomType(), gf->triangles.size(), gf->mesh_vertices.size());
 
   halfmesh.finish();
+
+  if(gf->getNumMeshElements() == 0){
+    Msg::Warning("Surface %d consists of no elements", gf->tag());
+  }
 }
 
 bool checkMeshCompound(GFaceCompound *gf, std::list<GEdge*> &edges)
