@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -269,7 +269,7 @@ int inCircumCircleAniso(GFace *gf, double *p1, double *p2, double *p3,
   const double d0 = (x[0] - uv[0]);
   const double d1 = (x[1] - uv[1]);
   const double d3 =  d0*d0*a + d1*d1*d + 2.0*d0*d1*b;
-  return d3 < Radius2;
+  return d3 < Radius2 - 1e-12;
 }
 
 int inCircumCircleAniso(GFace *gf, MTriangle *base,
@@ -828,7 +828,7 @@ bool insertVertex(bool force, GFace *gf, MVertex *v, double *param , MTri3 *t,
   else{
     recurFindCavityAniso(gf, shell, cavity, metric, param, t, data);
   }
-  
+
   return insertVertexB(shell, cavity, force, gf, v, param , t,
 		       allTets,
 		       activeTets,
@@ -1008,7 +1008,10 @@ static bool insertAPoint(GFace *gf,
     int index1 = data.getIndex(ptin->tri()->getVertex(1));
     int index2 = data.getIndex(ptin->tri()->getVertex(2));
     lc1 = (1. - uv[0] - uv[1]) * data.vSizes[index0] + uv[0] * data.vSizes[index1] + uv[1] * data.vSizes[index2];
-    lc = BGM_MeshSize(gf, center[0], center[1], p.x(), p.y(), p.z());
+    if (CTX::instance()->mesh.algo2d == ALGO_2D_BAMG)
+      lc = 1.;
+    else
+      lc = BGM_MeshSize(gf, center[0], center[1], p.x(), p.y(), p.z());
 
     //SMetric3 metr = BGM_MeshMetric(gf, center[0], center[1], p.x(), p.y(), p.z());
     //                               vMetricsBGM.push_back(metr);
@@ -1059,7 +1062,7 @@ void bowyerWatson(GFace *gf, int MAXPNT,
   //  if (equivalence)_printTris ("before.pos", AllTris.begin(), AllTris.end(), DATA);
   int nbSwaps = edgeSwapPass(gf, AllTris, SWCR_DEL, DATA);
   // _printTris ("after2.pos", AllTris, Us,Vs);
-  //  Msg::Debug("Delaunization of the initial mesh done (%d swaps)", nbSwaps);
+  Msg::Debug("Delaunization of the initial mesh done (%d swaps)", nbSwaps);
 
   if(AllTris.empty()){
     Msg::Error("No triangles in initial mesh");
@@ -1380,22 +1383,15 @@ void bowyerWatsonFrontal(GFace *gf,
   int ITERATION = 0;
   while (1){
     ++ITERATION;
-    /* if(ITERATION % 10== 0 && CTX::instance()->mesh.saveAll){
+    /*
+      if(ITERATION % 10== 0 && CTX::instance()->mesh.saveAll){
       char name[245];
       sprintf(name,"delFrontal_GFace_%d_Layer_%d.pos",gf->tag(),ITERATION);
       _printTris (name, AllTris.begin(), AllTris.end(), &DATA);
       sprintf(name,"delFrontal_GFace_%d_Layer_%d_Active.pos",gf->tag(),ITERATION);
       _printTris (name, ActiveTris.begin(), ActiveTris.end(), &DATA);
-      }*/
-    /* if(ITER % 100== 0){
-          char name[245];
-          sprintf(name,"delfr2d%d-ITER%d.pos",gf->tag(),ITER);
-          _printTris (name, AllTris, Us,Vs,true);
-	  //          sprintf(name,"delfr2dA%d-ITER%d.pos",gf->tag(),ITER);
-	  //          _printTris (name, ActiveTris, Us,Vs,false);
-        }
+      }
     */
-    //    printf("active_tris.size = %d\n",ActiveTris.size());
     if (!ActiveTris.size())break;
     MTri3 *worst = (*ActiveTris.begin());
     ActiveTris.erase(ActiveTris.begin());
@@ -1407,27 +1403,14 @@ void bowyerWatsonFrontal(GFace *gf,
                    gf->mesh_vertices.size(), worst->getRadius());
       double newPoint[2], metric[3];
       //optimalPointFrontal (gf,worst,active_edge,Us,Vs,vSizes,vSizesBGM,newPoint,metric);
-      if (optimalPointFrontalB (gf,worst,active_edge,DATA,newPoint,metric))
+      if (optimalPointFrontalB (gf,worst,active_edge,DATA,newPoint,metric)){
 	insertAPoint(gf, AllTris.end(), newPoint, metric, DATA, AllTris, &ActiveTris, worst);
-    }
-
-    /*   if(ITER % 1== 0){
-       char name[245];
-       sprintf(name,"frontal%d-ITER%d.pos",gf->tag(),ITER);
-       _printTris (name, AllTris, Us,Vs,false);
-     }
-    */
+      }
+    }    
   }
 
-
   nbSwaps = edgeSwapPass(gf, AllTris, SWCR_QUAL, DATA);
-  /*
-  char name[245];
-  sprintf(name,"frontal%d-real.pos", gf->tag());
-  _printTris (name, AllTris.begin(), AllTris.end(), DATA,false);
-  sprintf(name,"frontal%d-param.pos", gf->tag());
-  _printTris (name, AllTris.begin(), AllTris.end(), DATA,true);
-  */
+  
   transferDataStructure(gf, AllTris, DATA);
   //  removeThreeTrianglesNodes(gf);
 
