@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -128,8 +128,10 @@ void buildMeshGenerationDataStructures(GFace *gf,
     std::list<GVertex*> emb_vertx = gf->embeddedVertices();
     std::list<GVertex*>::iterator itvx = emb_vertx.begin();
     while(itvx != emb_vertx.end()){
-      MVertex *v = *((*itvx)->mesh_vertices.begin());
-      vSizesMap[v] = std::min(vSizesMap[v], (*itvx)->prescribedMeshSizeAtVertex());
+      if((*itvx)->mesh_vertices.size()){
+        MVertex *v = *((*itvx)->mesh_vertices.begin());
+        vSizesMap[v] = std::min(vSizesMap[v], (*itvx)->prescribedMeshSizeAtVertex());
+      }
       ++itvx;
     }
   }
@@ -260,26 +262,51 @@ void transferDataStructure(GFace *gf, std::set<MTri3*, compareTri3Ptr> &AllTris,
   // parameter space (it would be nicer to change the actual algorithm
   // to ensure that we create correctly-oriented triangles in the
   // first place)
+
+  // if BL triangles are considered, then all that is WRONG !
+
+
   if(gf->triangles.size() > 1){
+
+    bool BL = !gf->getColumns()->_toFirst.empty();
+
     double n1[3], n2[3];
     MTriangle *t = gf->triangles[0];
     MVertex *v0 = t->getVertex(0), *v1 = t->getVertex(1), *v2 = t->getVertex(2);
-    int index0 = data.getIndex (v0);
-    int index1 = data.getIndex (v1);
-    int index2 = data.getIndex (v2);
-    normal3points(data.Us[index0], data.Vs[index0], 0.,
-                  data.Us[index1], data.Vs[index1], 0.,
-                  data.Us[index2], data.Vs[index2], 0., n1);
+
+    if (!BL){
+      int index0 = data.getIndex (v0);
+      int index1 = data.getIndex (v1);
+      int index2 = data.getIndex (v2);
+      normal3points(data.Us[index0], data.Vs[index0], 0.,
+		    data.Us[index1], data.Vs[index1], 0.,
+		    data.Us[index2], data.Vs[index2], 0., n1);
+    }
+    else {
+      // BL --> PLANAR FACES !!!
+      normal3points(v0->x(),v0->y(),v0->z(),
+		    v1->x(),v1->y(),v1->z(),
+		    v2->x(),v2->y(),v2->z(), n1);
+    }
     for(unsigned int j = 1; j < gf->triangles.size(); j++){
       t = gf->triangles[j];
       v0 = t->getVertex(0); v1 = t->getVertex(1); v2 = t->getVertex(2);
-      index0 = data.getIndex (v0);
-      index1 = data.getIndex (v1);
-      index2 = data.getIndex (v2);
-      normal3points(data.Us[index0], data.Vs[index0], 0.,
-                    data.Us[index1], data.Vs[index1], 0.,
-                    data.Us[index2], data.Vs[index2], 0., n2);
+      if (!BL){
+	int index0 = data.getIndex (v0);
+	int index1 = data.getIndex (v1);
+	int index2 = data.getIndex (v2);
+	normal3points(data.Us[index0], data.Vs[index0], 0.,
+		      data.Us[index1], data.Vs[index1], 0.,
+		      data.Us[index2], data.Vs[index2], 0., n2);
+      }
+      else {
+	// BL --> PLANAR FACES !!!
+	normal3points(v0->x(),v0->y(),v0->z(),
+		      v1->x(),v1->y(),v1->z(),
+		      v2->x(),v2->y(),v2->z(), n2);
+      }
       double pp; prosca(n1, n2, &pp);
+      // orient the bignou
       if(pp < 0) t->reverse();
     }
   }
@@ -453,7 +480,8 @@ static int _removeTwoQuadsNodes(GFace *gf)
   std::set<MElement*>  touched;
   std::set<MVertex*>  vtouched;
   while (it != adj.end()) {
-    if(it->second.size()==2 && it->first->onWhat()->dim() == 2) {
+    MVertex *v = it->first;
+    if(it->second.size()==2 && v->onWhat()->dim() == 2) {
       MElement *q1 = it->second[0];
       MElement *q2 = it->second[1];
       if (q1->getNumVertices() == 4 &&
@@ -461,7 +489,7 @@ static int _removeTwoQuadsNodes(GFace *gf)
           touched.find(q1) == touched.end() && touched.find(q2) == touched.end()){
         int comm = 0;
         for (int i=0;i<4;i++){
-          if (q1->getVertex(i) == it->first){
+          if (q1->getVertex(i) == v){
             comm = i;
             break;
           }
@@ -471,8 +499,8 @@ static int _removeTwoQuadsNodes(GFace *gf)
         MVertex *v3 = q1->getVertex((comm+3)%4);
         MVertex *v4 = 0;
         for (int i=0;i<4;i++){
-          if (q2->getVertex(i) != v1 && q2->getVertex(i) != v2 &&
-              q2->getVertex(i) != v3 && q2->getVertex(i) != it->first){
+          if (q2->getVertex(i) != v1 && q2->getVertex(i) != v3 &&
+              q2->getVertex(i) != v){
             v4 = q2->getVertex(i);
             break;
           }
@@ -484,8 +512,8 @@ static int _removeTwoQuadsNodes(GFace *gf)
           return 0;
         }
         MQuadrangle *q = new MQuadrangle(v1,v2,v3,v4);
-        double s1 = surfaceFaceUV(q,gf);
-        double s2 = surfaceFaceUV(q1,gf) + surfaceFaceUV(q2,gf);;
+        double s1 = 0;//surfaceFaceUV(q,gf);
+        double s2 = 1;//surfaceFaceUV(q1,gf) + surfaceFaceUV(q2,gf);;
         if (s1 > s2){
           delete q;
         }
@@ -493,13 +521,14 @@ static int _removeTwoQuadsNodes(GFace *gf)
           touched.insert(q1);
           touched.insert(q2);
           gf->quadrangles.push_back(q);
-          vtouched.insert(it->first);
+          vtouched.insert(v);
         }
       }
     }
     it++;
   }
   std::vector<MQuadrangle*> quadrangles2;
+  quadrangles2.reserve(gf->quadrangles.size() - touched.size());
   for(unsigned int i = 0; i < gf->quadrangles.size(); i++){
     if(touched.find(gf->quadrangles[i]) == touched.end()){
       quadrangles2.push_back(gf->quadrangles[i]);
@@ -511,6 +540,7 @@ static int _removeTwoQuadsNodes(GFace *gf)
   gf->quadrangles = quadrangles2;
 
   std::vector<MVertex*> mesh_vertices2;
+  mesh_vertices2.reserve(gf->mesh_vertices.size() - vtouched.size());
   for(unsigned int i = 0; i < gf->mesh_vertices.size(); i++){
     if(vtouched.find(gf->mesh_vertices[i]) == vtouched.end()){
       mesh_vertices2.push_back(gf->mesh_vertices[i]);
@@ -536,6 +566,66 @@ int removeTwoQuadsNodes(GFace *gf)
   return nbRemove;
 }
 
+
+static bool _tryToCollapseThatVertex2 (GFace *gf,
+				       std::vector<MElement*> &e1,
+				       std::vector<MElement*> &e2,
+				       MElement *q,
+				       MVertex *v1,
+				       MVertex *v2)
+{
+  std::vector<MElement*> e = e1;
+  e.insert(e.end(), e2.begin(), e2.end());
+
+  double x1 = v1->x();
+  double y1 = v1->y();
+  double z1 = v1->z();
+
+  double x2 = v2->x();
+  double y2 = v2->y();
+  double z2 = v2->z();
+
+  // new position of v1 && v2
+  double initialGuess[2]={0,0};
+  GPoint pp = gf->closestPoint(SPoint3(0.5*(x1+x2),0.5*(y1+y2),0.5*(z1+z2)),initialGuess);
+
+  //  double surface_old = 0;
+  //  double surface_new = 0;
+  double worst_quality_old = 1.0;
+  double worst_quality_new = 1.0;
+
+  //  surface_old = surfaceFaceUV(q,gf,false);
+  int count=0;
+  for (unsigned int j=0;j<e.size();++j){
+    if (e[j] != q){
+      count++;
+      worst_quality_old = std::min(worst_quality_old,e[j]-> etaShapeMeasure());
+      v1->x() = pp.x();v1->y() = pp.y();v1->z() = pp.z();
+      v2->x() = pp.x();v2->y() = pp.y();v2->z() = pp.z();
+      worst_quality_new = std::min(worst_quality_new,e[j]-> etaShapeMeasure());
+      v1->x() = x1;v1->y() = y1;v1->z() = z1;
+      v2->x() = x2;v2->y() = y2;v2->z() = z2;
+    }
+  }
+
+  //  printf("%d %g %g %g %g\n", count, surface_old, surface_new, worst_quality_old , worst_quality_new);
+
+  if (worst_quality_new >  worst_quality_old ) {
+    v1->x() = pp.x();v1->y() = pp.y();v1->z() = pp.z();
+    for (unsigned int j=0;j<e.size();++j){
+      if (e[j] != q){
+        for (int k=0;k<4;k++){
+          if (e[j]->getVertex(k) == v2){
+            e[j]->setVertex(k,v1);
+          }
+        }
+      }
+    }
+    return true;
+  }
+  return false;
+}
+
 // collapse v1 & v2 to their middle and replace into e1 & e2
 static bool _tryToCollapseThatVertex (GFace *gf,
                                       std::vector<MElement*> &e1,
@@ -548,7 +638,9 @@ static bool _tryToCollapseThatVertex (GFace *gf,
   e.insert(e.end(), e2.begin(), e2.end());
 
   double uu1,vv1;
-  v1->getParameter(0,uu1);
+  if (!v1->getParameter(0,uu1)){
+    return _tryToCollapseThatVertex2 (gf,e1,e2,q,v1,v2);
+  }
   v1->getParameter(1,vv1);
   double x1 = v1->x();
   double y1 = v1->y();
@@ -682,7 +774,7 @@ static int _removeDiamonds(GFace *gf)
           v2->onWhat()->dim() == 2 &&
           v3->onWhat()->dim() == 2 &&
           v4->onWhat()->dim() == 2 &&
-          it1->second.size() ==3 &&  it3->second.size() == 3 &&
+          it1->second.size() == 3 && it3->second.size() == 3 &&
           _tryToCollapseThatVertex (gf, it1->second, it3->second,
                                       q, v1, v3)){
         touched.insert(v1);
@@ -822,14 +914,11 @@ void getAllBoundaryLayerVertices (GFace *gf, std::set<MVertex*> &vs){
   vs.clear();
   BoundaryLayerColumns* _columns = gf->getColumns();
   if (!_columns)return;
-  for ( std::map<MElement*,std::vector<MElement*> >::iterator it = _columns->_elemColumns.begin();
-	it != _columns->_elemColumns.end();it++){
-    std::vector<MElement *> &e = it->second;
-    for (unsigned int i=0;i<e.size();i++){
-      for (int j=0;j<e[i]->getNumVertices();j++){
-	vs.insert(e[i]->getVertex(j));
-      }
-    }
+  for ( std::multimap<MVertex*, BoundaryLayerData>::iterator it = _columns->_data.begin();
+	it != _columns->_data.end();it++){
+    BoundaryLayerData &data = it->second;
+    for (size_t i = 0;i<data._column.size();i++)
+      vs.insert(data._column[i]);
   }
 }
 
@@ -980,14 +1069,19 @@ bool edgeSwap(std::set<swapquad> &configs, MTri3 *t1, GFace *gf, int iLocalEdge,
                           std::min(lc1, lcBGM1) : lcBGM1, 0, &data, gf);
   MTri3 *t2b3 = new MTri3(t2b, Extend1dMeshIn2dSurfaces() ?
                           std::min(lc2, lcBGM2) : lcBGM2, 0, &data, gf);
+  //  printf("%d %d %d -- %d %d %d edge %d\n",t1->tri()->getVertex(0)->getNum(),t1->tri()->getVertex(1)->getNum(),t1->tri()->getVertex(2)->getNum(),
+  //	 t2->tri()->getVertex(0)->getNum(),t2->tri()->getVertex(1)->getNum(),t2->tri()->getVertex(2)->getNum(),iLocalEdge);
+  //  printf("%d %d %d \n",v2->getNum(),v3->getNum(),v4->getNum());
+  //  printf("%d %d %d \n",v4->getNum(),v3->getNum(),v1->getNum());
 
-  cavity.push_back(t1b3);
   cavity.push_back(t2b3);
+  cavity.push_back(t1b3);
   t1->setDeleted(true);
   t2->setDeleted(true);
   connectTriangles(cavity);
-  newTris.push_back(t1b3);
   newTris.push_back(t2b3);
+  newTris.push_back(t1b3);
+  //  printf("%d %d\n",t1b3->isDeleted(),t2b3->isDeleted());
 
   return true;
 }
@@ -996,30 +1090,43 @@ int edgeSwapPass(GFace *gf, std::set<MTri3*, compareTri3Ptr> &allTris,
 		 const swapCriterion &cr,bidimMeshData & data)
 {
   typedef std::set<MTri3*, compareTri3Ptr> CONTAINER;
-  
+
   int nbSwapTot = 0;
   std::set<swapquad> configs;
-  for(int iter = 0; iter < 1200; iter++){
+
+  std::set<MTri3*, compareTri3Ptr> allTris2;
+
+  for(int iter = 0; iter < 10; iter++){
     int nbSwap = 0;
     std::vector<MTri3*> newTris;
-    for(CONTAINER::iterator it = allTris.begin(); it != allTris.end(); ++it){
-      if(!(*it)->isDeleted()){
+    CONTAINER::iterator it = allTris.begin();
+    while(it != allTris.end()){
+      CONTAINER::iterator current = it++;
+      if(!(*current)->isDeleted()){
 	for(int i = 0; i < 3; i++){
-	  if(edgeSwap(configs, *it, gf, i, newTris, cr, data)){
+	  if(edgeSwap(configs, *current, gf, i, newTris, cr, data)){
 	    nbSwap++;
 	    break;
 	  }
 	}
       }
       else{
-	delete *it;
-	CONTAINER::iterator itb = it;
-	++it;
-	allTris.erase(itb);
-	if(it == allTris.end()) break;
+	delete (*current)->tri();
+	delete *current;
+	allTris.erase(current);
       }
     }
+
+    //    allTris = allTris2;
+
     allTris.insert(newTris.begin(), newTris.end());
+
+    // for(CONTAINER::iterator it = allTris.begin(); it != allTris.end(); ++it){
+    //   printf("---> %d %d %d (%d)\n",(*it)->tri()->getVertex(0)->getNum(),
+    // 	     (*it)->tri()->getVertex(1)->getNum(),
+    // 	     (*it)->tri()->getVertex(2)->getNum(),(*it)->isDeleted() );
+    // }
+
     nbSwapTot += nbSwap;
     if(nbSwap == 0) break;
   }
@@ -1144,6 +1251,7 @@ static int _recombineIntoQuads(GFace *gf, double minqual, bool cubicGraph = 1)
   std::sort(pairs.begin(),pairs.end());
   std::set<MElement*> touched;
 
+
   if(CTX::instance()->mesh.algoRecombine != 0){
 #if defined(HAVE_BLOSSOM)
     int ncount = gf->triangles.size();
@@ -1190,6 +1298,7 @@ static int _recombineIntoQuads(GFace *gf, double minqual, bool cubicGraph = 1)
         }
       }
 
+
       double matzeit = 0.0;
       char MATCHFILE[256];
       sprintf(MATCHFILE,".face.match");
@@ -1251,6 +1360,7 @@ static int _recombineIntoQuads(GFace *gf, double minqual, bool cubicGraph = 1)
 #endif
   }
 
+
   std::vector<RecombineTriangle>::iterator itp = pairs.begin();
   while(itp != pairs.end()){
     // recombine if difference between max quad angle and right
@@ -1259,6 +1369,7 @@ static int _recombineIntoQuads(GFace *gf, double minqual, bool cubicGraph = 1)
     if(itp->angle < gf->meshAttributes.recombineAngle){
       MElement *t1 = itp->t1;
       MElement *t2 = itp->t2;
+
       if(touched.find(t1) == touched.end() &&
           touched.find(t2) == touched.end()){
         touched.insert(t1);
@@ -1296,6 +1407,7 @@ static int _recombineIntoQuads(GFace *gf, double minqual, bool cubicGraph = 1)
   }
   gf->triangles = triangles2;
 
+
   if(CTX::instance()->mesh.algoRecombine != 1){
     quadsToTriangles(gf, minqual);
   }
@@ -1331,7 +1443,7 @@ void recombineIntoQuads(GFace *gf,
   double t1 = Cpu();
 
   bool haveParam = true;
-  bool saveAll = false; //CTX::instance()->mesh.saveAll;
+  bool saveAll = CTX::instance()->mesh.saveAll;
   if(gf->geomType() == GEntity::DiscreteSurface && !gf->getCompound())
     haveParam = false;
 
@@ -1339,10 +1451,10 @@ void recombineIntoQuads(GFace *gf,
   int success = _recombineIntoQuads(gf, minqual);
 
   if (saveAll) gf->model()->writeMSH("raw.msh");
+
   if(haveParam && nodeRepositioning){
     RelocateVertices (gf,CTX::instance()->mesh.nbSmoothing);
   }
-
   // blossom-quad algo
   if(success && CTX::instance()->mesh.algoRecombine != 0){
     if(topologicalOpti){
@@ -1359,7 +1471,7 @@ void recombineIntoQuads(GFace *gf,
           //          printStats (gf, "toto");
           if (ITER > 20) break;
           ITER ++;
-        }
+	}
       }
     }
     quadsToTriangles(gf, minqual);
@@ -1400,12 +1512,12 @@ void quadsToTriangles(GFace *gf, double minqual)
       double qual1 = std::min(t11->gammaShapeMeasure(),t12->gammaShapeMeasure());
       double qual2 = std::min(t21->gammaShapeMeasure(),t22->gammaShapeMeasure());
 
-      double surf1 = surfaceFaceUV(t11,gf) + surfaceFaceUV(t12,gf);
-      double surf2 = surfaceFaceUV(t21,gf) + surfaceFaceUV(t22,gf);
+      //      double surf1 = surfaceFaceUV(t11,gf) + surfaceFaceUV(t12,gf);
+      //      double surf2 = surfaceFaceUV(t21,gf) + surfaceFaceUV(t22,gf);
 
       int option = 0;
-      if (surf1 > surf2 * (1.+1.e-8))option = 2;
-      else if (surf2 > surf1 * (1.+1.e-8))option = 1;
+      //      if (surf1 > surf2 * (1.+1.e-8))option = 2;
+      //      else if (surf2 > surf1 * (1.+1.e-8))option = 1;
 
       if (option == 1 || (option == 0 && qual1 > qual2)){
         gf->triangles.push_back(t11);
