@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -106,7 +106,7 @@ bool buildEdgeCavity(MTet4 *t, int iLocalEdge, MVertex **v1, MVertex **v2,
     }
     // FIXME when hybrid mesh, this loops for ever
     if (cavity.size() > 1000) {
-      printf("cavity size gets laaaarge\n");
+      //      printf("cavity size gets laaaarge\n");
       return false;
     }
     //    printf("%d %d\n",ITER++, cavity.size());
@@ -128,10 +128,22 @@ void BuildSwapPattern3(SwapPattern *sc)
   sc->trianguls = trgul ;
 }
 
+/*
+   0             1
+   +------------+
+   |            |
+   |            |
+   |            |
+   +------------+ 
+   3            2
+
+*/
+
+
 void BuildSwapPattern4(SwapPattern *sc)
 {
   static int trgl[][3] =
-    { {0,1,2}, {0,2,3}, {0,1,3}, {1,2,3} };
+    { {0,1,2}, {2,3,0}, {1,2,3}, {3,0,1} };
   static int trgul[][5] =
     { {0,1,-1,-1,-1}, {2,3,-1,-1,-1} };
 
@@ -206,6 +218,11 @@ bool edgeSwap(std::vector<MTet4 *> &newTets,
               int iLocalEdge,
               const qmTetrahedron::Measures &cr)
 {
+
+  //static int edges[6][2] =    {{0,1},{0,2},{0,3},{1,2},{1,3},{2,3}};
+  int permut[6] = {0,3,1,2,5,4};
+  iLocalEdge = permut[iLocalEdge];
+ 
   std::vector<MTet4*> cavity;
   std::vector<MTet4*> outside;
   std::vector<MVertex*> ring;
@@ -277,6 +294,7 @@ bool edgeSwap(std::vector<MTet4 *> &newTets,
 
   // if there exist no swap that enhance the quality
   if (best <= tetQualityRef) return false;
+  // does random swaps  if (best < .01) return false;
 
   // we have the best configuration, so we swap
   // printf("outside size = %d\n",outside.size());
