@@ -1,8 +1,9 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
 
+#include "robustPredicates.h"
 #include "qualityMeasures.h"
 #include "BDS.h"
 #include "MVertex.h"
@@ -636,17 +637,13 @@ double qmTetrahedron::eta(const double &x1, const double &y1, const double &z1,
                           const double &x4, const double &y4, const double &z4,
                           double *volume)
 {
-  double mat[3][3];
-  mat[0][0] = x2 - x1;
-  mat[0][1] = x3 - x1;
-  mat[0][2] = x4 - x1;
-  mat[1][0] = y2 - y1;
-  mat[1][1] = y3 - y1;
-  mat[1][2] = y4 - y1;
-  mat[2][0] = z2 - z1;
-  mat[2][1] = z3 - z1;
-  mat[2][2] = z4 - z1;
-  *volume = fabs(det3x3(mat)) / 6.;
+  double p0[3] = {x1, y1, z1};
+  double p1[3] = {x2, y2, z2};
+  double p2[3] = {x3, y3, z3};
+  double p3[3] = {x4, y4, z4};
+
+  *volume =fabs (robustPredicates::orient3d (p0,p1,p2,p3)) / 6.0;
+
   double l = ((x2 - x1) * (x2 - x1) +
               (y2 - y1) * (y2 - y1) +
               (z2 - z1) * (z2 - z1));
@@ -665,40 +662,46 @@ double qmTetrahedron::gamma(const double &x1, const double &y1, const double &z1
                             const double &x4, const double &y4, const double &z4,
                             double *volume)
 {
-  double mat[3][3];
-  mat[0][0] = x2 - x1;
-  mat[0][1] = x3 - x1;
-  mat[0][2] = x4 - x1;
-  mat[1][0] = y2 - y1;
-  mat[1][1] = y3 - y1;
-  mat[1][2] = y4 - y1;
-  mat[2][0] = z2 - z1;
-  mat[2][1] = z3 - z1;
-  mat[2][2] = z4 - z1;
-  *volume = fabs(det3x3(mat)) / 6.;
   double p0[3] = {x1, y1, z1};
   double p1[3] = {x2, y2, z2};
   double p2[3] = {x3, y3, z3};
   double p3[3] = {x4, y4, z4};
+
+  *volume =fabs (robustPredicates::orient3d (p0,p1,p2,p3)) / 6.0;
+  
+  // double mat[3][3];
+  // mat[0][0] = x2 - x1;
+  // mat[0][1] = x3 - x1;
+  // mat[0][2] = x4 - x1;
+  // mat[1][0] = y2 - y1;
+  // mat[1][1] = y3 - y1;
+  // mat[1][2] = y4 - y1;
+  // mat[2][0] = z2 - z1;
+  // mat[2][1] = z3 - z1;
+  // mat[2][2] = z4 - z1;
+  // *volume = fabs(det3x3(mat)) / 6.;
+
+  // printf("vv %g volume %g\n",vv,*volume);
+
   double s1 = fabs(triangle_area(p0, p1, p2));
   double s2 = fabs(triangle_area(p0, p2, p3));
   double s3 = fabs(triangle_area(p0, p1, p3));
   double s4 = fabs(triangle_area(p1, p2, p3));
   double rhoin = 3. * fabs(*volume) / (s1 + s2 + s3 + s4);
-  double l = sqrt((x2 - x1) * (x2 - x1) +
-                  (y2 - y1) * (y2 - y1) +
-                  (z2 - z1) * (z2 - z1));
-  l = std::max(l, sqrt((x3 - x1) * (x3 - x1) + (y3 - y1) * (y3 - y1) +
-                       (z3 - z1) * (z3 - z1)));
-  l = std::max(l, sqrt((x4 - x1) * (x4 - x1) + (y4 - y1) * (y4 - y1) +
-                       (z4 - z1) * (z4 - z1)));
-  l = std::max(l, sqrt((x3 - x2) * (x3 - x2) + (y3 - y2) * (y3 - y2) +
-                       (z3 - z2) * (z3 - z2)));
-  l = std::max(l, sqrt((x4 - x2) * (x4 - x2) + (y4 - y2) * (y4 - y2) +
-                       (z4 - z2) * (z4 - z2)));
-  l = std::max(l, sqrt((x3 - x4) * (x3 - x4) + (y3 - y4) * (y3 - y4) +
-                       (z3 - z4) * (z3 - z4)));
-  return 2. * sqrt(6.) * rhoin / l;
+  double l = (x2 - x1) * (x2 - x1) +
+    (y2 - y1) * (y2 - y1) +
+    (z2 - z1) * (z2 - z1);
+  l = std::max(l, ((x3 - x1) * (x3 - x1) + (y3 - y1) * (y3 - y1) +
+		   (z3 - z1) * (z3 - z1)));
+  l = std::max(l, ((x4 - x1) * (x4 - x1) + (y4 - y1) * (y4 - y1) +
+		   (z4 - z1) * (z4 - z1)));
+  l = std::max(l, ((x3 - x2) * (x3 - x2) + (y3 - y2) * (y3 - y2) +
+		   (z3 - z2) * (z3 - z2)));
+  l = std::max(l, ((x4 - x2) * (x4 - x2) + (y4 - y2) * (y4 - y2) +
+		   (z4 - z2) * (z4 - z2)));
+  l = std::max(l, ((x3 - x4) * (x3 - x4) + (y3 - y4) * (y3 - y4) +
+		   (z3 - z4) * (z3 - z4)));
+  return sqrt(24./l) * rhoin ;
 }
 
 
