@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -323,7 +323,9 @@ CondNumBasis::CondNumBasis(int tag, int cnOrder) :
   }
   const bool serendip = false;
 
-  FuncSpaceData data = ( ElementType::ParentTypeFromTag(tag) == TYPE_PYR )? FuncSpaceData(true, tag, true, _condNumOrder+1, _condNumOrder, &serendip) : FuncSpaceData(true, tag, _condNumOrder, &serendip);
+  FuncSpaceData data = (ElementType::ParentTypeFromTag(tag) == TYPE_PYR) ?
+      FuncSpaceData(true, tag, true, 1, _condNumOrder-1, &serendip) :
+      FuncSpaceData(true, tag, _condNumOrder, &serendip);
 
   fullMatrix<double> lagPoints;                                  // Sampling points
   gmshGeneratePoints(data, lagPoints);
@@ -386,7 +388,7 @@ int CondNumBasis::condNumOrder(int parentType, int order)
     case TYPE_TET : return (order == 1) ? 0 : order;
     case TYPE_PRI : return order;
     case TYPE_HEX : return order;
-    case TYPE_PYR : return (order == 1) ? 0 : order;
+    case TYPE_PYR : return order;
     case TYPE_TRIH : return 0;
     default :
       Msg::Error("Unknown element type %d, return order 0", parentType);
@@ -405,7 +407,7 @@ inline void CondNumBasis::getInvCondNumGeneral(int nCondNumNodes,
                                                const fullMatrix<double> &nodesXYZ,
                                                const fullMatrix<double> &normals,
                                                fullVector<double> &condNum) const
-{ 
+{
   switch (_dim) {
 
     case 0 : {
@@ -448,7 +450,7 @@ inline void CondNumBasis::getInvCondNumGeneral(int nCondNumNodes,
         condNum(i) = calcInvCondNum3D<sign>(dxdX, dxdY, dxdZ,
                                             dydX, dydY, dydZ,
                                             dzdX, dzdY, dzdZ);
-     }
+      }
       break;
     }
   }
