@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -185,4 +185,7 @@ void transformPointsIntoOrthoBasis(const std::vector<SPoint3> &ptsProj,
                                    std::vector<SPoint3> &pointsUV,
 				   const SPoint3 &ptCG, const mean_plane &meanPlane);
 
+bool catenary(double x0, double x1, double y0, double y1, double ys, int N,
+              double *yp);
+
 #endif
