@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -10,9 +10,9 @@
 #include "Context.h"
 #include "GmshMessage.h"
 #include "FaultZone.h"
-
-#include "Geo.h"
 #include "GModel.h"
+#include "GModelIO_GEO.h"
+#include "Geo.h"
 #include "SVector3.h"
 #include "GEdge.h"
 #include "GFace.h"
@@ -104,7 +104,7 @@ PView *GMSH_FaultZonePlugin::execute(PView *view)
     return view;
   }
 
-  std::list< GEdge* > embeddedEdges = gFace->embeddedEdges();
+  std::list<GEdge*> embeddedEdges = gFace->embeddedEdges();
   std::list<GEdge*>::const_iterator itl;
   for(itl = embeddedEdges.begin();itl != embeddedEdges.end(); ++itl)
     if ((*itl)->length() != 0) break;
@@ -636,6 +636,8 @@ void GMSH_FaultZoneMesher::CreateJointElements(GModel* gModel, GFace* gFace,
   }
 
   // replace physical edges by physical surfaces
+
+#if 1 // FIXME this is wrong !?!
   for(int i = 0; i < List_Nbr(gModel->getGEOInternals()->PhysicalGroups); i++){
     PhysicalGroup *p = *(PhysicalGroup**)List_Pointer
       (GModel::current()->getGEOInternals()->PhysicalGroups, i);
@@ -653,9 +655,10 @@ void GMSH_FaultZoneMesher::CreateJointElements(GModel* gModel, GFace* gFace,
       if (List_Nbr(faceEntities) > 0){
         std::stringstream sufix;
         sufix << p->Num;
-        int num = gModel->setPhysicalName
-          (prefix+sufix.str(), 2, ++GModel::current()->getGEOInternals()->MaxPhysicalNum);
-        PhysicalGroup *pnew = Create_PhysicalGroup(num, MSH_PHYSICAL_SURFACE, faceEntities);
+        int t = GModel::current()->getGEOInternals()->getMaxPhysicalTag();
+        GModel::current()->getGEOInternals()->setMaxPhysicalTag(t + 1);
+        int num = gModel->setPhysicalName(prefix+sufix.str(), 2, t + 1);
+        PhysicalGroup *pnew = CreatePhysicalGroup(num, MSH_PHYSICAL_SURFACE, faceEntities);
         List_Add(gModel->getGEOInternals()->PhysicalGroups, &pnew);
         for(int j = 0; j < List_Nbr(faceEntities); j++){
           int num;
@@ -670,6 +673,8 @@ void GMSH_FaultZoneMesher::CreateJointElements(GModel* gModel, GFace* gFace,
       List_Delete(faceEntities);
     }
   }
+#endif
+
 }
 
 //================================================================================
@@ -803,4 +808,3 @@ void GMSH_FaultZoneMesher::ModifyJointNodePosition(double eps){
     mVert->z() += vect.z();
   }
 }
-
