@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -152,7 +152,7 @@ PView *GMSH_TetrahedralizePlugin::execute(PView *v)
 
 PView *GMSH_TetrahedralizePlugin::execute(PView *v)
 {
-  Msg::Error("Plugin(Tetrahedralize requires mesh module");
+  Msg::Error("Plugin(Tetrahedralize) requires mesh module");
   return v;
 }
 
