@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -462,18 +462,22 @@ class globalVTKData {
       it->clear();
     }
     vtkGlobalConnectivity.clear();
+    std::vector<vectInt>().swap(vtkGlobalConnectivity);
   }
   static void clearGlobalCellType()
   {
     vtkGlobalCellType.clear();
+    std::vector<int>().swap(vtkGlobalCellType);
   }
   static void clearGlobalCoords()
   {
     vtkGlobalCoords.clear();
+    std::vector<PCoords>().swap(vtkGlobalCoords);
   }
   static void clearGlobalValues()
   {
     vtkGlobalValues.clear();
+    std::vector<PValues>().swap(vtkGlobalValues);
   }
   static void clearGlobalData()
   {
