@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@onelab.info>.
@@ -622,6 +622,14 @@ class dofManager : public dofManagerBase{
       R.push_back(it->first);
     }
   }
+	virtual void getFixedDof(std::set<Dof>& R)
+	{
+		R.clear();
+		typename std::map<Dof, dataVec>::iterator it;
+		for(it = fixed.begin(); it != fixed.end(); ++it){
+      R.insert(it->first);
+    }
+	}
 
   virtual int getDofNumber(const Dof& key)
   {
